@@ -1,7 +1,7 @@
 /*
  * File: IO.c
  *
- * Copyright (C) 2000, 2001, 2002 Jorge Arellano Cid <jcid@inf.utfsm.cl>
+ * Copyright (C) 2000, 2001, 2002 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <sys/socket.h>
 #include <glib.h>
 #include <gdk/gdk.h>
+#include "../msg.h"
 #include "../chain.h"
 #include "../list.h"
 #include "../klist.h"
@@ -49,6 +50,24 @@
 static Klist_t *ValidIOs = NULL; /* Active IOs list. It holds pointers to
                                   * IOData_t structures. */
 
+/*
+ * Debug procedure...
+
+static void IO_print_cond_status(gchar  *str,  GIOCondition  cond,
+                                 GIOChannel *gio, gint key)
+{
+   MSG("%s FD=%d key=%d [", str, g_io_channel_unix_get_fd(gio), key);
+   MSG(cond & G_IO_IN ? "G_IO_IN " : "");
+   MSG(cond & G_IO_OUT ? "G_IO_OUT " : "");
+   MSG(cond & G_IO_PRI ? "G_IO_PRI " : "");
+   MSG(cond & G_IO_ERR ? "G_IO_ERR " : "");
+   MSG(cond & G_IO_HUP ? "G_IO_HUP " : "");
+   MSG(cond & G_IO_NVAL ? "G_IO_NVAL " : "");
+   MSG("]\n");
+}
+ */
+
+
 /* IO API  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 /*
@@ -206,18 +225,18 @@ static void *IO_write_chunk(void *ptr)
    /* switch to detached state */
    pthread_detach(data->thrID);
 
-   //g_print("thr::\n");
-   //g_print("      [thrID:%lu]\n", (gulong)data->thrID);
+   _MSG("thr::\n");
+   _MSG("      [thrID:%lu]\n", (gulong)data->thrID);
 
    while (1) {
-      //g_print("thr:: trying to lock mutex\n");
+      _MSG("thr:: trying to lock mutex\n");
       if (!lock) {
          pthread_mutex_lock(&data->mut);
          lock = 1;
       }
-      //g_print("thr:: mutex locked!\n");
-      //g_print("thr:: Buf1:%p Buf2:%p LastChunk:%d Done:%d\n",
-      //        data->Buf1, data->Buf2, data->LastChunk, data->Done);
+      _MSG("thr:: mutex locked!\n");
+      _MSG("thr:: Buf1:%p Buf2:%p LastChunk:%d Done:%d\n",
+           data->Buf1, data->Buf2, data->LastChunk, data->Done);
       if (data->Buf2) {
          /* Buf1 := Buf2;  Buf2 := NULL */
          g_free(data->Buf1Start);
@@ -225,17 +244,17 @@ static void *IO_write_chunk(void *ptr)
          data->Buf1Size = data->Buf2Size;
          data->Buf2 = NULL;
          data->Buf2Size = 0;
-         //g_print("thr:: Buf1:%p Buf2:%p LastChunk:%d Done:%d\n",
-         //        data->Buf1, data->Buf2, data->LastChunk, data->Done);
+         _MSG("thr:: Buf1:%p Buf2:%p LastChunk:%d Done:%d\n",
+              data->Buf1, data->Buf2, data->LastChunk, data->Done);
          pthread_mutex_unlock(&data->mut);
          lock = 0;
-         //g_print("thr:: mutex unlocked!\n");
+         _MSG("thr:: mutex unlocked!\n");
 
          /*** write all ***/
          done = 0;
          while (!done) {
             St = write(data->FD, data->Buf1, data->Buf1Size);
-            //g_print("thr:: St=%d\n", St);
+            _MSG("thr:: St=%d\n", St);
             if (St < 0) {
                if (errno == EINTR) {
                   continue;
@@ -243,7 +262,7 @@ static void *IO_write_chunk(void *ptr)
                   perror("IO_write_chunk");
                   return NULL;
                }
-            } else if (St < data->Buf1Size) {
+            } else if ((size_t)St < data->Buf1Size) {
                /* Not all data written */
                data->Buf1 = (gchar *)data->Buf1 + St;
                data->Buf1Size -= St;
@@ -268,11 +287,11 @@ static void *IO_write_chunk(void *ptr)
          while (st < 0 && errno == EINTR);
          data->Done = 1;
          pthread_mutex_unlock(&data->mut);
-         //g_print("thr:: LastChunk:%d Done:%d  --Bailing out!\n",
-         //        data->LastChunk, data->Done);
+         _MSG("thr:: LastChunk:%d Done:%d  --Bailing out!\n",
+              data->LastChunk, data->Done);
          return NULL;
       } else {
-         //g_print("thr:: going to cond_wait...\n");
+         _MSG("thr:: going to cond_wait...\n");
          pthread_cond_wait(&data->cond, &data->mut);
          lock = 1;
       }
@@ -368,6 +387,9 @@ static void IO_close_fd(IOData_t *io, gint CloseCode)
    /* Remove this IOData_t reference, from our ValidIOs list
     * We don't deallocate it here, just remove from the list.*/
    IO_del(io);
+
+   /* Stop the polling on this FD */
+   g_source_remove(io->watch_id);
 }
 
 /*
@@ -415,7 +437,7 @@ static gboolean IO_read(IOData_t *io)
          IO_close_fd(io, IO_StopRd);
          a_IO_ccc(OpEnd, 2, FWD, io->Info, io, NULL);
 
-      } else if ( St < io->BufSize ){
+      } else if ( (size_t)St < io->BufSize ){
          /* We have all the new data */
          a_IO_ccc(OpSend, 2, FWD, io->Info, io, NULL);
          ret = TRUE;
@@ -437,12 +459,13 @@ static gboolean IO_read(IOData_t *io)
 static gboolean IO_write(IOData_t *io)
 {
    ssize_t St;
-   gboolean ret;
+   gboolean ret, DataPending;
 
    DEBUG_MSG(3, "  IO_write\n");
 
-   while (1) {
+   do {
       ret = FALSE;
+      DataPending = FALSE;
       St = write(io->FD, io->Buf, io->BufSize);
       io->Status = St;
 
@@ -457,12 +480,12 @@ static gboolean IO_write(IOData_t *io)
          } else if (errno == EAGAIN) {
             DEBUG_MSG(4, "  IO_write: EAGAIN\n");
             ret = TRUE;
-            break;
          }
-      } else if ( St < io->BufSize ){
+      } else if ( (size_t)St < io->BufSize ){
          /* Not all data written */
          io->BufSize -= St;
          io->Buf = (gchar *)io->Buf + St;
+         DataPending = TRUE;
          DEBUG_MSG(4, "  IO_write: Changing io->Buf (%d)\n", St);
 
       } else {
@@ -480,9 +503,8 @@ static gboolean IO_write(IOData_t *io)
                a_IO_ccc(OpEnd, 1, FWD, io->Info, io, NULL);
             }
          }
-         break;
       }
-   }/*while*/
+   } while (DataPending);
 
    return ret;
 }
@@ -497,11 +519,12 @@ static gboolean IO_callback(GIOChannel *src, GIOCondition cond, gpointer data)
    gint io_key = GPOINTER_TO_INT(data);
    IOData_t *io = IO_get(io_key);
 
-   DEBUG_MSG(3, " IO_callback: [GIOcond %d]\n", cond);
+   // IO_print_cond_status("IO_callback: ", cond, src, io_key);
 
-   /* Sometimes glib delivers events on already aborted FDs  --Jcid */
+   /* There should be no more glib events on already closed FDs  --Jcid */
    if ( io == NULL ) {
-      DEBUG_MSG(3, " IO_callback: call on already closed io!\n");
+      g_warning("IO_callback: call on already closed io!\n");
+      g_assert_not_reached();
       return FALSE;
    }
 
@@ -546,13 +569,15 @@ static void IO_submit(IOData_t *r_io)
          FD_CLOEXEC | fcntl(r_io->FD, F_GETFD));
 
    if ( r_io->Op == IORead ) {
-      g_io_add_watch(r_io->GioCh, G_IO_IN | G_IO_ERR | G_IO_HUP,
-                     IO_callback, GINT_TO_POINTER (r_io->Key));
+      r_io->watch_id =
+         g_io_add_watch(r_io->GioCh, G_IO_IN | G_IO_ERR | G_IO_HUP,
+                        IO_callback, GINT_TO_POINTER (r_io->Key));
       g_io_channel_unref(r_io->GioCh);
 
    } else if ( r_io->Op == IOWrite || r_io->Op == IOWrites ) {
-      g_io_add_watch(r_io->GioCh, G_IO_OUT | G_IO_ERR,
-                     IO_callback, GINT_TO_POINTER (r_io->Key));
+      r_io->watch_id =
+         g_io_add_watch(r_io->GioCh, G_IO_OUT | G_IO_ERR,
+                        IO_callback, GINT_TO_POINTER (r_io->Key));
       g_io_channel_unref(r_io->GioCh);
    }
 }
@@ -646,7 +671,7 @@ void a_IO_ccc(int Op, int Branch, int Dir, ChainLink *Info,
             IO_free(io);
             break;
          case OpAbort:
-            g_print(" Not implemented\n");
+            MSG(" Not implemented\n");
             break;
          }
       }
