@@ -2,7 +2,7 @@
  * File: about.c
  *
  * Copyright (C) 1997 Raph Levien <raph@acm.org>
- * Copyright (C) 1999, 2001 Jorge Arellano Cid <jcid@inf.utfsm.cl>
+ * Copyright (C) 1999, 2001 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,6 +12,7 @@
 
 #include <pthread.h>
 
+#include <config.h>
 #include "Url.h"
 #include "../nav.h"
 #include "../web.h"
@@ -30,10 +31,10 @@ struct _SplashInfo {
 static char *Splash=
 "Content-type: text/html\n"
 "\n"
-"<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.0 Transitional//EN'>\n"
+"<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN'>\n"
 "<html>\n"
 "<head>\n"
-"<title>Splash screen for dillo-0.7.3</title>\n"
+"<title>Splash screen for dillo-" VERSION "</title>\n"
 "</head>\n"
 "<body bgcolor='#778899' text='#000000' link='#000000' vlink='#000000'>\n"
 "\n"
@@ -50,7 +51,7 @@ static char *Splash=
 "     <table width='100%' border='0' bgcolor='#ffffff'>\n"
 "     <tr>\n"
 "       <td valign='top' align='left'>\n"
-"         <h1>&nbsp;Welcome to Dillo 0.7.3&nbsp;</h1>\n"
+"         <h1>&nbsp;Welcome to Dillo " VERSION "&nbsp;</h1>\n"
 "       </td>\n"
 "     </tr>\n"
 "     </table>\n"
@@ -140,17 +141,13 @@ static char *Splash=
 "    <tr>\n"
 "      <td bgcolor='#FFFFFF'>\n"
 "        <table border='0' cellspacing='0' cellpadding='5'><tr><td>\n"
-"        <table border='0' cellspacing='' cellpadding='2'><tr>\n"
+"        <table border='0' cellpadding='2'><tr>\n"
 "        <td>&nbsp;&nbsp;</td>\n"
 "        <td>\n"
 "         <a href='http://lwn.net'>LWN</a></td>\n"
 "        </tr><tr>\n"
 "        <td>&nbsp;&nbsp;</td>\n"
 "        <td>\n"
-"         <a href='http://linuxtoday.com/'>Linux Today</a></td>\n"
-"        </tr><tr>\n"
-"        <td>&nbsp;&nbsp;</td>\n"
-"        <td>\n"
 "         <a href='http://slashdot.org/'>Slashdot</a></td>\n"
 "        </tr><tr>\n"
 "        <td>&nbsp;&nbsp;</td>\n"
@@ -163,6 +160,14 @@ static char *Splash=
 "        </tr><tr>\n"
 "        <td>&nbsp;&nbsp;</td>\n"
 "        <td>\n"
+"         <a href='http://linuxtoday.com/'>Linux Today</a></td>\n"
+"        </tr><tr>\n"
+"        <td>&nbsp;&nbsp;</td>\n"
+"        <td>\n"
+"         <a href='http://www.gnu-darwin.org/update.html'>Monster News</a></td>\n"
+"        </tr><tr>\n"
+"        <td>&nbsp;&nbsp;</td>\n"
+"        <td>\n"
 "         <a href='http://www.theregister.co.uk/index.html'>The Register</a></td>\n"
 "        </tr><tr>\n"
 "        <td>&nbsp;&nbsp;</td>\n"
@@ -196,11 +201,14 @@ static char *Splash=
 "    <tr>\n"
 "      <td bgcolor='#FFFFFF'>\n"
 "        <table border='0' cellspacing='0' cellpadding='5'><tr><td>\n"
-"        <table border='0' cellspacing='' cellpadding='2'><tr>\n"
+"        <table border='0' cellpadding='2'><tr>\n"
 "        <td>&nbsp;&nbsp;</td>\n"
 "        <td><a href='http://www.google.com/'>Google</a></td>\n"
 "        </tr><tr>\n"
 "        <td>&nbsp;&nbsp;</td>\n"
+"        <td><a href='http://www.wikipedia.org/'>Wikipedia</a></td>\n"
+"        </tr><tr>\n"
+"        <td>&nbsp;&nbsp;</td>\n"
 "        <td><a href='http://freshmeat.net/'>FreshMeat</a></td>\n"
 "        </tr><tr>\n"
 "        <td>&nbsp;&nbsp;</td>\n"
@@ -233,7 +241,7 @@ static char *Splash=
 "    <tr>\n"
 "      <td bgcolor='#FFFFFF'>\n"
 "        <table border='0' cellspacing='0' cellpadding='5'><tr><td>\n"
-"        <table border='0' cellspacing='' cellpadding='2'><tr>\n"
+"        <table border='0' cellpadding='2'><tr>\n"
 "        <td>&nbsp;&nbsp;</td>\n"
 "        <td><a href='http://www.violence.de'>Peace&amp;Violence</a></td>\n"
 "        </tr></table>\n"
@@ -298,7 +306,7 @@ static char *Splash=
 "<tr>\n"
 "  <td bgcolor='#CCCCCC'>\n"
 "    <h4>Release overview</h4>\n"
-"    03 August 2003\n"
+"    08 February 2004\n"
 "  </td>\n"
 "</tr><tr>\n"
 "  <td bgcolor='#FFFFFF'>\n"
@@ -306,14 +314,31 @@ static char *Splash=
 "    <tr>\n"
 "     <td>\n"
 "\n"
-"<P>\n"
-"This is mainly a bug-fix release: the main fixes\n"
-"were on improving selection code, much faster rendering of\n"
-"huge pages, a bug fix in findtext, and better exception\n"
-"handling inside I/O.\n"
-"<P>\n"
-"Plus the polished eye-candy that the new toolbar icons bring!\n"
-"<P>\n"
+"<p>\n"
+" After six months,\n"
+"the new dillo-0.8.0 comes with the largest set of improvements\n"
+"ever:\n"
+"<p>\n"
+" The dpi framework was redesigned. Now the communications and\n"
+"managing are done with a daemon: <em>dpid</em>. We also developed some\n"
+"plugins: downloads, ftp, hello, bookmarks. Downloads uses <em>wget</em>\n"
+"and it can handle several connections at the same time!\n"
+"<p>\n"
+" This release also introduces the <em>bug-meter</em>: a button that shows\n"
+"the count of detected HTML errors in a page, also providing the\n"
+"line where they occur and a hint of how to solve them.\n"
+"It's intended to help webmasters and page\n"
+"authors to polish the contents of their sites with a view to\n"
+"make them standards compliant.\n"
+"<p>\n"
+" Some interesting additions: a context menu for images, a\n"
+"web-search dialog, the \"-geometry\" switch in the CLI, lots of\n"
+"improvements inside the internal widget system, portability enhancements\n"
+"and the usual bug fixes.\n"
+"<p>\n"
+" These improvements, plus much more detailed in the ChangeLog,\n"
+"round up our best dillo!\n"
+"<p>\n"
 "Remember that dillo project uses a release model where every new\n"
 "browser shall be better than the former.\n"
 "<EM>Keep up with the latest one!</EM>\n"
@@ -342,14 +367,26 @@ static char *Splash=
 "    <table border='0' cellspacing='0' cellpadding='5'>\n"
 "    <tr>\n"
 "     <td>\n"
-" <ul>\n"
-"   <li>Improved the speed when selecting large text areas.</li>\n"
-"   <li>Fixed a wrong implementation of incremental resizing.</li>\n"
-"   <li>Made jumping to anchors faster.</li>\n"
-"   <li>Made a new set of toolbar icons!</li>\n"
-"   <li>Fixed a bug that permeated findtext.</li>\n"
-"   <li>Added the missing EINTR handlers in IO.c and file.c.</li>\n"
-" </ul>\n"
+"<ul>\n"
+"<li>Introduced a new way of handling dillo plugins! Now the\n"
+"   communications and managing is done by a daemon: dpid.\n"
+"   This comes with a lot of advantages described in Dpid.txt.\n"
+"<li>Developed a plugin for downloads. It uses wget and can handle several\n"
+"   connections at the same time.\n"
+"<li>Adapted the generic parser to make HTML error detection, providing\n"
+"   the line number and a hint (expected tag) in the error message!\n"
+"<li>Added a bug-meter button that shows the count of detected HTML errors\n"
+"   (left click shows the errors, right click offers a validation menu).\n"
+"<li>Added a right-mouse-button popup for images!\n"
+"<li>Added a web search dialog (with toolbar icon, shortcut is Ctrl-S).\n"
+"   The search engine can be set in dillorc (defaults to google).\n"
+"<li>Improved configure.in for better portability.\n"
+"<li>Made SPAM-safe URLs aware of image buttons.\n"
+"<li>Made images showing the ALT text as long as no image data has been\n"
+"   retrieved.\n"
+"<li>Added the \"-geometry\" switch to the CLI.\n"
+"<li>Fixed a bug in findtext (wrong highlighting).\n"
+"</ul>\n"
 "     </td>\n"
 "    </tr></table>\n"
 "  </td>\n"
@@ -382,7 +419,8 @@ static char *Splash=
 "     dir  within  the  tarball;  you can find directions on everything\n"
 "     else at the home page.\n"
 "   <li> Dillo has context sensitive menus using the\n"
-"     right mouse button (available on pages, links, and the Back and Forward buttons).\n"
+"     right mouse button (available on pages, links, images,\n"
+"     the Back and Forward buttons, and bug meter).\n"
 "   <li> Dillo behaves very nicely when browsing local files, images, and HTML.\n"
 "     It's also very good for Internet searching (try Google!).\n"
 "   <li> This release is mainly intended <strong>for developers</strong>\n"
@@ -420,7 +458,6 @@ static char *Splash=
 "<!-- end of the main layout table -->\n"
 "<!--                              -->\n"
 "\n"
-"</td>\n"
 "</tr>\n"
 "</table>\n"
 "\n"
@@ -500,7 +537,7 @@ static gint About_get(ChainLink *Info, void *Data1, void *Data2)
    else
       loc = "http://www.google.com/";
 
-   LocUrl = a_Url_new(loc, NULL, 0, 0);
+   LocUrl = a_Url_new(loc, NULL, 0, 0, 0);
    a_Nav_push(web->bw, LocUrl);
    a_Url_free(LocUrl);
    return -1;
@@ -521,7 +558,7 @@ void a_About_ccc(int Op, int Branch, int Dir, ChainLink *Info,
       if (Dir == BCK) {
          switch (Op) {
          case OpStart:
-            /* (Data1 = URL;  Data2 = Web) */
+            /* (Data1 = Url;  Data2 = Web) */
             // Info->LocalKey gets set in About_get
             if ((FD = About_get(Info, Data1, Data2)) == -1)
                a_Chain_fcb(OpAbort, Info, NULL, NULL);
