@@ -1,7 +1,7 @@
 /*
  * File: file.c :)
  *
- * Copyright (C) 2000, 2001 Jorge Arellano Cid <jcid@inf.utfsm.cl>
+ * Copyright (C) 2000, 2001 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,10 @@
 #include "../web.h"
 #include "../interface.h"
 
+#define DEBUG_LEVEL 5
+#include "../debug.h"
+
+
 typedef struct _DilloDir {
    gint FD_Write, FD_Read;
    char *dirname;
@@ -301,7 +305,7 @@ static gint File_ext(const char *filename, const char *ext)
 
    if ( !(e = strrchr(filename, '.')) )
       return 0;
-   return (strcasecmp(ext, ++e) == 0);
+   return (g_strcasecmp(ext, ++e) == 0);
 }
 
 /*
@@ -601,7 +605,7 @@ static GString *File_dir2html(DilloDir *Ddir)
             ++Ddir->flist_size;
          }
       }
-      /* sort the enries */
+      /* sort the entries */
       qsort(Ddir->dlist, Ddir->dlist_size, sizeof(char *), File_comp);
       qsort(Ddir->flist, Ddir->flist_size, sizeof(char *), File_comp);
 
@@ -645,7 +649,7 @@ static void File_not_found_msg(DilloWeb *web, const char *filename, int fd)
       a_Interface_msg(web->bw, "ERROR: Can't find %s %s",
                          (fd == -2) ? "" : "file", filename);
    else
-      g_print("Warning: Can't find <%s>\n", filename);
+      DEBUG_MSG(4, "Warning: Can't find <%s>\n", filename);
 }
 
 /*
@@ -661,6 +665,7 @@ void a_File_ccc(int Op, int Branch, int Dir, ChainLink *Info,
       if (Dir == BCK) {
          switch (Op) {
          case OpStart:
+            /* (Data1 = Url;  Data2 = Web) */
             File_get(Info, Data1, Data2);
             break;
          }
