@@ -1,7 +1,7 @@
 /*
  * File: http.c
  *
- * Copyright (C) 2000, 2001 Jorge Arellano Cid <jcid@inf.utfsm.cl>
+ * Copyright (C) 2000, 2001 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,9 +43,11 @@
 #endif
 
 /* Used to send a message to the bw's status bar */
-#define BW_MSG(web, root, fmt...) \
-   (a_Web_valid(web) && (!(root) || (web)->flags & WEB_RootUrl)) ? \
-   a_Interface_msg((web)->bw, fmt) : (root)
+#define BW_MSG(web, root, fmt...)                                     \
+G_STMT_START {                                                        \
+   if (a_Web_valid((web)) && (!(root) || (web)->flags & WEB_RootUrl)) \
+      a_Interface_msg((web)->bw, fmt);                                \
+} G_STMT_END
 
 #define DEBUG_LEVEL 5
 #include "../debug.h"
@@ -84,7 +86,7 @@ gint a_Http_init(void)
    gchar *env_proxy = getenv("http_proxy");
 
    if (env_proxy && strlen(env_proxy))
-      HTTP_Proxy = a_Url_new(env_proxy, NULL, 0, 0);
+      HTTP_Proxy = a_Url_new(env_proxy, NULL, 0, 0, 0);
    if (!HTTP_Proxy && prefs.http_proxy)
       HTTP_Proxy = a_Url_dup(prefs.http_proxy);
    return 0;
@@ -217,15 +219,18 @@ static gboolean
    Info = S->Info;
    if ( cond & G_IO_HUP ) {
       DEBUG_MSG(3, "--Connection broken\n");
+      /* get rid of S->GioCh */
       g_io_channel_close(S->GioCh);
+      g_io_channel_unref(S->GioCh);
+      S->GioCh = NULL;
       if (S->addr_list_iter->next) {
          S->addr_list_iter = S->addr_list_iter->next;
-         if (Http_connect_socket(Info) == 0)
-            return FALSE;
+         if (Http_connect_socket(Info) < 0) {
+            BW_MSG(S->web, 0, "ERROR: unable to connect to remote host");
+            Http_socket_free(SKey);
+            a_Chain_fcb(OpAbort, Info, NULL, NULL);
+         }
       }
-      BW_MSG(S->web, 0, "ERROR: unable to connect to remote host");
-      a_Chain_fcb(OpAbort, Info, NULL, NULL);
-      Http_socket_free(SKey);
    } else if ( S->Err ) {
       BW_MSG(S->web, 0, "ERROR: %s", g_strerror(S->Err));
       DEBUG_MSG(3, "Http_use_socket ERROR: %s\n", g_strerror(S->Err));
@@ -249,6 +254,7 @@ static gboolean
 static int Http_connect_socket(ChainLink *Info)
 {
    gint status;
+   guint watch_id;
 #ifdef ENABLE_IPV6
    struct sockaddr_in6 name;
 #else
@@ -307,10 +313,13 @@ static int Http_connect_socket(ChainLink *Info)
 
    if (!S->GioCh)
       S->GioCh = g_io_channel_unix_new(S->SockFD);
-   g_io_add_watch(S->GioCh, G_IO_ERR | G_IO_HUP | G_IO_OUT,
-                  Http_use_socket, Info->LocalKey);
+   watch_id = g_io_add_watch(S->GioCh, G_IO_ERR | G_IO_HUP | G_IO_OUT,
+                             Http_use_socket, Info->LocalKey);
    status = connect(S->SockFD, (struct sockaddr *)&name, socket_len);
    if ( status == -1 && errno != EINPROGRESS ) {
+      DEBUG_MSG(5, "Http_connect_socket ERROR: %s\n", g_strerror(errno));
+      g_io_channel_close(S->GioCh);
+      g_source_remove(watch_id);
       S->Err = errno;
       return -1;
    }
@@ -506,8 +515,8 @@ void a_Http_ccc(int Op, int Branch, int Dir, ChainLink *Info,
             /* unlink IO_Info */
             if (S) {
                a_Chain_del_link(Info, BCK);
-               a_Chain_fcb(OpAbort, Info, NULL, NULL);
                Http_socket_free(SKey);
+               a_Chain_fcb(OpAbort, Info, NULL, NULL);
             }
             break;
          }
