@@ -1,7 +1,7 @@
 /*
  * File: cache.c
  *
- * Copyright 2000, 2001 Jorge Arellano Cid <jcid@inf.utfsm.cl>
+ * Copyright 2000, 2001 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 
+#include "msg.h"
 #include "list.h"
 #include "IO/Url.h"
 #include "IO/IO.h"
@@ -338,7 +339,7 @@ static gint Cache_prepare_reload(DilloUrl *url)
       Cache_entry_remove(NULL, url);
       return 1;
    } else {
-      g_print("Cache_prepare_reload: ERROR, entry still has clients\n");
+      MSG("Cache_prepare_reload: ERROR, entry still has clients\n");
    }
 
    return 0;
@@ -357,7 +358,14 @@ static gint Cache_open_url(DilloWeb *Web, CA_Callback_t Call, void *CbData)
    DilloUrl *Url = Web->url;
    CacheData_t *entry = Cache_entry_search(Url);
 
-   if ( !entry ) {
+   _MSG("Cache_open_url: %s %sFOUND\n", URL_STR(Url), entry ? "" : "NOT ");
+
+   if ( entry ) {
+      /* URL is cached: feed our client with cached data */
+      ClientKey = Cache_client_enqueue(entry->Url, Web, Call, CbData);
+      Cache_delayed_process_queue(entry);
+
+   } else {
       /* URL not cached: create an entry, send our client to the queue,
        * and open a new connection */
       entry = Cache_entry_add(Url);
@@ -373,10 +381,6 @@ static gint Cache_open_url(DilloWeb *Web, CA_Callback_t Call, void *CbData)
          a_Cache_ccc(OpAbort, 1, FWD, entry->CCCQuery, NULL, NULL);
          ClientKey = 0; /* aborted */
       }
-   } else {
-      /* Feed our client with cached data */
-      ClientKey = Cache_client_enqueue(entry->Url, Web, Call, CbData);
-      Cache_delayed_process_queue(entry);
    }
    return ClientKey;
 }
@@ -430,10 +434,10 @@ gint a_Cache_open_url(void *web, CA_Callback_t Call, void *CbData)
 }
 
 /*
- * Return the pointer to URL 'Data' in the cache.
- * (We also return its size in 'Size' and take care of redirections)
+ * Get the pointer to the URL document, and its size, from the cache entry.
+ * Return: 1 cached, 0 not cached.
  */
-char *a_Cache_url_read(const DilloUrl *Url, gint *Size)
+gint a_Cache_get_buf(const DilloUrl *Url, gchar **PBuf, gint *BufSize)
 {
    CacheData_t **stack = NULL;
    gint stackSize = 0;
@@ -460,10 +464,11 @@ char *a_Cache_url_read(const DilloUrl *Url, gint *Size)
          }
       }
    } while (entry);
-
    g_free (stack);
-   *Size = (entry) ? entry->ValidSize : 0;
-   return (entry) ? (char *) entry->Data : NULL;
+
+   *BufSize = (entry) ? entry->ValidSize : 0;
+   *PBuf = (entry) ? (gchar *) entry->Data : NULL;
+   return (entry ? 1 : 0);
 }
 
 /*
@@ -546,14 +551,17 @@ static void Cache_parse_header(CacheData_t *entry, IOData_t *io, gint HdrLen)
    GList *Cookies;
 #endif
 
-   if ( header[9] == '3' && header[10] == '0' ) {
+   if ( HdrLen < 12 ) {
+      /* Not enough info. */
+
+   } if ( header[9] == '3' && header[10] == '0' ) {
       /* 30x: URL redirection */
       entry->Flags |= CA_Redirect;
       if ( header[11] == '1' )
          /* 301 Moved Permanently */
          entry->Flags |= CA_ForceRedirect;
       location_str = Cache_parse_field(header, "Location");
-      entry->Location = a_Url_new(location_str, URL_STR_(entry->Url), 0, 0);
+      entry->Location = a_Url_new(location_str, URL_STR_(entry->Url), 0, 0, 0);
       g_free(location_str);
 
    } else if ( strncmp(header + 9, "404", 3) == 0 ) {
@@ -598,7 +606,7 @@ static void Cache_parse_header(CacheData_t *entry, IOData_t *io, gint HdrLen)
 
    /* Get Content-Type */
    if ( (Type = Cache_parse_field(header, "Content-Type")) == NULL ) {
-     DEBUG_HTTP_MSG("Server didn't send Content-Type in header.\n");
+     MSG_HTTP("Server didn't send Content-Type in header.\n");
      Type = g_strdup(a_Misc_get_content_type_from_data(
                         entry->Data, entry->ValidSize));
    }
@@ -658,8 +666,8 @@ static void Cache_process_io(int Op, void *VPtr)
 
    if ( Op == IOClose ) {
       if (entry->Flags & CA_GotLength && entry->TotalSize != entry->ValidSize){
-         DEBUG_HTTP_MSG("Content-Length does NOT match message body,\n"
-                        " at: %s\n", URL_STR_(entry->Url));
+         MSG_HTTP("Content-Length does NOT match message body,\n"
+                  " at: %s\n", URL_STR_(entry->Url));
       }
       entry->Flags |= CA_GotData;
       entry->Flags &= ~CA_Stopped;          /* it may catch up! */
@@ -735,7 +743,8 @@ static gint Cache_redirect(CacheData_t *entry, gint Flags, BrowserWindow *bw)
       if ( Flags & WEB_RootUrl ) {
          /* Redirection of the main page */
          a_Nav_remove_top_url(bw);
-         NewUrl =a_Url_new(URL_STR_(entry->Location),URL_STR_(entry->Url),0,0);
+         NewUrl = a_Url_new(URL_STR_(entry->Location), URL_STR_(entry->Url),
+                            0, 0, 0);
          a_Nav_push(bw, NewUrl);
          a_Url_free(NewUrl);
       } else {
@@ -847,6 +856,7 @@ static gint Cache_delayed_process_queue_callback(gpointer data)
  */
 static void Cache_delayed_process_queue(CacheData_t *entry)
 {
+   /* no need to track the idle_id, the function always removes itself */
    gtk_idle_add((GtkFunction)Cache_delayed_process_queue_callback, entry);
 }
 
@@ -984,6 +994,7 @@ void a_Cache_ccc(int Op, int Branch, int Dir, ChainLink *Info,
          /* Answering branch */
          switch (Op) {
          case OpStart:
+            /* Data2 = entry->Url */
             Info->LocalKey = Data2;
             if ((entry = Cache_entry_search(Info->LocalKey))) {
                entry->CCCAnswer = Info;
@@ -1033,7 +1044,7 @@ void a_Cache_ccc(int Op, int Branch, int Dir, ChainLink *Info,
             break;
             }
          case OpStop:
-            g_print(" Not implemented\n");
+            MSG(" Not implemented\n");
             break;
          case OpAbort:
             Cache_entry_remove_raw(NULL, Info->LocalKey);
