@@ -50,7 +50,7 @@ struct _CacheClient {
  */
 void a_Cache_init(void);
 gint a_Cache_open_url(void *Web, CA_Callback_t Call, void *CbData);
-char *a_Cache_url_read(const DilloUrl *url, gint *size);
+gint a_Cache_get_buf(const DilloUrl *Url, gchar **PBuf, gint *BufSize);
 void a_Cache_freeall(void);
 void a_Cache_null_client(int Op, CacheClient_t *Client);
 void a_Cache_stop_client(gint Key);
