@@ -1,7 +1,7 @@
 /*
  * File: dns.c
  *
- * Copyright (C) 1999, 2000, 2001 Jorge Arellano Cid <jcid@inf.utfsm.cl>
+ * Copyright (C) 1999, 2000, 2001 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include <signal.h>
 #include <string.h>
 
+#include "msg.h"
 #include "dns.h"
 #include "list.h"
 
@@ -80,7 +81,7 @@ typedef struct {
 /*
  * Forward declarations
  */
-static gint Dns_timeout_client(gpointer data);
+static gboolean Dns_timeout_client(gpointer data);
 
 /*
  * Local Data
@@ -148,10 +149,10 @@ void Dns_queue_print()
 {
    gint i;
 
-   g_print("Queue: [");
+   MSG("Queue: [");
    for (i = 0; i < dns_queue_size; i++)
-      g_print("%d:%s ", dns_queue[i].channel, dns_queue[i].hostname);
-   g_print("]\n");
+      MSG("%d:%s ", dns_queue[i].channel, dns_queue[i].hostname);
+   MSG("]\n");
 }
  */
 
@@ -207,7 +208,7 @@ void a_Dns_init(void)
       dns_server[i].ip_ready = FALSE;
       dns_server[i].addr_list = NULL;
       dns_server[i].hostname = NULL;
-      dns_server[i].timeout_id = -1;
+      dns_server[i].timeout_id = 0;
 #ifdef G_DNS_THREADED
       dns_server[i].th1 = (pthread_t) -1;
 #endif
@@ -464,7 +465,7 @@ static void Dns_assign_channels(void)
  * This is a Gtk+ timeout function that
  * reads the DNS results and resumes the stopped jobs.
  */
-static gint Dns_timeout_client(gpointer data)
+static gboolean Dns_timeout_client(gpointer data)
 {
    gint channel = GPOINTER_TO_INT(data);
    DnsServer *srv = &dns_server[channel];
@@ -476,6 +477,7 @@ static gint Dns_timeout_client(gpointer data)
       }
       Dns_serve_channel(channel);
       Dns_assign_channels();
+      srv->timeout_id = 0;
       return FALSE;  /* Done! */
    }
 
