@@ -2,7 +2,7 @@
  * File: dw_image.c
  *
  * Copyright (C) 2001 Sebastian Geerken  <S.Geerken@ping.de>,
- *                    Jorge Arellano Cid <jcid@inf.utfsm.cl>
+ *                    Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -10,6 +10,7 @@
  * (at your option) any later version.
  */
 
+#include "msg.h"
 #include "dw_image.h"
 #include "dw_gtk_viewport.h"
 #include "prefs.h"
@@ -33,22 +34,22 @@ static void Dw_image_size_allocate      (DwWidget *widget,
 static void Dw_image_draw               (DwWidget *widget,
                                          DwRectangle *area,
                                          GdkEventExpose *event);
-static gint Dw_image_button_press       (DwWidget *widget,
+static gboolean Dw_image_button_press   (DwWidget *widget,
                                          gint32 x,
                                          gint32 y,
                                          GdkEventButton *event);
-static gint Dw_image_button_release     (DwWidget *widget,
+static gboolean Dw_image_button_release (DwWidget *widget,
                                          gint32 x,
                                          gint32 y,
                                          GdkEventButton *event);
-static gint Dw_image_motion_notify      (DwWidget *widget,
+static gboolean Dw_image_motion_notify  (DwWidget *widget,
                                          gint32 x,
                                          gint32 y,
                                          GdkEventMotion *event);
-static gint Dw_image_enter_notify       (DwWidget *widget,
+static gboolean Dw_image_enter_notify   (DwWidget *widget,
                                          DwWidget *last_widget,
                                          GdkEventMotion *event);
-static gint Dw_image_leave_notify       (DwWidget *widget,
+static gboolean Dw_image_leave_notify   (DwWidget *widget,
                                          DwWidget *next_widget,
                                          GdkEventMotion *event);
 static DwIterator* Dw_image_iterator    (DwWidget *widget,
@@ -146,11 +147,13 @@ static void Dw_image_init (DwImage *image)
    image->url = NULL;
    image->width = 0;
    image->height = 0;
+   image->alt_text_width = -1; /* not yet calculated */
    image->buffer = NULL;
    image->scaled_buffer = NULL;
    image->tooltip = NULL;
    image->alt_text = NULL;
    image->usemap_url = NULL;
+   image->ismap = FALSE;
    for (i = 0; i < DW_HIGHLIGHT_NUM_LAYERS; i++)
       image->selected[i] = FALSE;
 }
@@ -171,37 +174,37 @@ static void Dw_image_class_init (DwImageClass *klass)
 
    image_signals[LINK_ENTERED] =
       gtk_signal_new ("link_entered",
-                      GTK_RUN_FIRST,
+                      GTK_RUN_LAST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwImageClass, link_entered),
                       p_Dw_marshal_link_enter,
-                      GTK_TYPE_NONE,
+                      GTK_TYPE_BOOL,
                       3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT);
    image_signals[LINK_PRESSED] =
       gtk_signal_new ("link_pressed",
-                      GTK_RUN_FIRST,
+                      GTK_RUN_LAST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwImageClass, link_pressed),
                       p_Dw_marshal_link_button,
-                      GTK_TYPE_NONE,
+                      GTK_TYPE_BOOL,
                       4, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT,
                       GTK_TYPE_GDK_EVENT);
    image_signals[LINK_RELEASED] =
       gtk_signal_new ("link_released",
-                      GTK_RUN_FIRST,
+                      GTK_RUN_LAST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwImageClass, link_released),
                       p_Dw_marshal_link_button,
-                      GTK_TYPE_NONE,
+                      GTK_TYPE_BOOL,
                       4, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT,
                       GTK_TYPE_GDK_EVENT);
    image_signals[LINK_CLICKED] =
       gtk_signal_new ("link_clicked",
-                      GTK_RUN_FIRST,
+                      GTK_RUN_LAST,
                       object_class->type,
                       GTK_SIGNAL_OFFSET (DwImageClass, link_clicked),
                       p_Dw_marshal_link_button,
-                      GTK_TYPE_NONE,
+                      GTK_TYPE_BOOL,
                       4, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT,
                       GTK_TYPE_GDK_EVENT);
    gtk_object_class_add_signals (object_class, image_signals, LAST_SIGNAL);
@@ -250,11 +253,25 @@ static void Dw_image_size_request (DwWidget *widget,
    DwImage *image;
 
    image = DW_IMAGE (widget);
-   requisition->width =
-      image->width + p_Dw_style_box_diff_width (widget->style);
-   requisition->ascent =
-      image->height + p_Dw_style_box_diff_height (widget->style);
-   requisition->descent = 0;
+   
+   if (image->buffer != NULL ||
+       (image->alt_text == NULL || image->alt_text[0] == '\0')) {
+      requisition->width = image->width;
+      requisition->ascent = image->height;
+      requisition->descent = 0;
+   } else {
+      if (image->alt_text_width == -1)
+         image->alt_text_width = 
+            gdk_string_width (widget->style->font->font, image->alt_text);
+
+      requisition->width = image->alt_text_width;
+      requisition->ascent = widget->style->font->font->ascent;
+      requisition->descent = widget->style->font->font->descent;
+   }
+
+   requisition->width += p_Dw_style_box_diff_width (widget->style);
+   requisition->ascent += p_Dw_style_box_offset_y (widget->style);
+   requisition->descent += p_Dw_style_box_rest_height (widget->style);
 }
 
 
@@ -268,8 +285,7 @@ static void Dw_image_size_allocate (DwWidget *widget,
 
    /* if image is moved only */
    if (allocation->width == widget->allocation.width &&
-       allocation->ascent + allocation->descent
-       == widget->allocation.ascent + widget->allocation.descent)
+       allocation->ascent + allocation->descent == DW_WIDGET_HEIGHT(widget))
       return;
 
    /* this is also done in a_Dw_widget_size_allocate, but
@@ -297,18 +313,13 @@ static void Dw_image_draw (DwWidget *widget,
    int i;
    gboolean selected = FALSE;
 
-   //g_print (">Area x=%d y=%d w=%d h=%d\n", area->x, area->y,
-   //         area->width, area->height);
-
-   p_Dw_widget_draw_widget_box (widget, area, FALSE);
-
    if (image->buffer) {
+      p_Dw_widget_draw_widget_box (widget, area, FALSE);
+
       content.x = p_Dw_style_box_offset_x (widget->style);
       content.y = p_Dw_style_box_offset_y (widget->style);
-      content.width =
-         widget->allocation.width - p_Dw_style_box_diff_width (widget->style);
-      content.height = widget->allocation.ascent + widget->allocation.descent -
-         p_Dw_style_box_diff_height (widget->style);
+      content.width = DW_WIDGET_CONTENT_WIDTH(widget);
+      content.height = DW_WIDGET_CONTENT_HEIGHT(widget);
 
       if (p_Dw_rectangle_intersect (area, &content, &intersection)) {
          vx = p_Dw_widget_x_world_to_viewport (widget, widget->allocation.x);
@@ -331,6 +342,27 @@ static void Dw_image_draw (DwWidget *widget,
             intersection.width, intersection.height, GDK_RGB_DITHER_MAX,
             bstart, content.width * 3);
       }
+   } else {
+      if (image->alt_text && image->alt_text[0]) {
+         if (image->alt_text_width == -1)
+            image->alt_text_width = 
+               gdk_string_width (widget->style->font->font, image->alt_text);
+
+         if (widget->allocation.width < image->alt_text_width ||
+             widget->allocation.ascent + widget->allocation.descent
+             < widget->style->font->font->ascent
+             + widget->style->font->font->descent)
+            p_Dw_widget_will_clip (widget);
+
+         p_Dw_widget_draw_widget_box (widget, area, FALSE);
+         gdk_draw_string
+            (DW_WIDGET_WINDOW (widget), widget->style->font->font,
+             widget->style->color->gc,
+             p_Dw_widget_x_world_to_viewport (widget, widget->allocation.x),
+             p_Dw_widget_y_world_to_viewport (widget, widget->allocation.y)
+             + widget->style->font->font->ascent,
+             image->alt_text);
+      }
    }
 
    for (i = 0; i < DW_HIGHLIGHT_NUM_LAYERS && !selected; i++)
@@ -343,9 +375,9 @@ static void Dw_image_draw (DwWidget *widget,
 /*
  * Standard Dw function.
  */
-static gint Dw_image_enter_notify (DwWidget *widget,
-                                   DwWidget *last_widget,
-                                   GdkEventMotion *event)
+static gboolean Dw_image_enter_notify (DwWidget *widget,
+                                       DwWidget *last_widget,
+                                       GdkEventMotion *event)
 {
    DwImage *image = DW_IMAGE (widget);
 
@@ -358,11 +390,12 @@ static gint Dw_image_enter_notify (DwWidget *widget,
 /*
  * Standard Dw function.
  */
-static gint Dw_image_leave_notify (DwWidget *widget,
-                                   DwWidget *next_widget,
-                                   GdkEventMotion *event)
+static gboolean Dw_image_leave_notify (DwWidget *widget,
+                                       DwWidget *next_widget,
+                                       GdkEventMotion *event)
 {
    DwImage *image = DW_IMAGE (widget);
+   gboolean return_val = FALSE;
 
    if (image->tooltip)
       a_Dw_tooltip_on_leave (image->tooltip);
@@ -370,7 +403,8 @@ static gint Dw_image_leave_notify (DwWidget *widget,
    if (image->hover_link != -1) {
       image->hover_link = -1;
       gtk_signal_emit (GTK_OBJECT (widget), image_signals[LINK_ENTERED],
-                       -1, -1, -1);
+                       -1, -1, -1, &return_val);
+      return return_val;
    }
 
    return FALSE;
@@ -380,35 +414,37 @@ static gint Dw_image_leave_notify (DwWidget *widget,
 /*
  * Standard Dw function.
  */
-static gint Dw_image_button_press (DwWidget *widget,
-                                   gint32 x,
-                                   gint32 y,
-                                   GdkEventButton *event)
+static gboolean Dw_image_button_press (DwWidget *widget,
+                                       gint32 x,
+                                       gint32 y,
+                                       GdkEventButton *event)
 {
    DwImage *image = DW_IMAGE (widget);
    gint link_x, link_y;
+   gboolean return_val = FALSE;
 
    Dw_image_find_link (image, x, y, &image->pressed_link, &link_x, &link_y);
 
-   if (image->pressed_link >= 0) {
+   if (image->pressed_link >= 0)
       gtk_signal_emit (GTK_OBJECT (widget), image_signals[LINK_PRESSED],
-                       image->pressed_link, link_x, link_y, event);
-      return TRUE;
-   } else
-      return FALSE;
+                       image->pressed_link, link_x, link_y, event,
+                       &return_val);
+
+   return return_val;
 }
 
 
 /*
  * Standard Dw function.
  */
-static gint Dw_image_button_release (DwWidget *widget,
-                                     gint32 x,
-                                     gint32 y,
-                                     GdkEventButton *event)
+static gboolean Dw_image_button_release (DwWidget *widget,
+                                         gint32 x,
+                                         gint32 y,
+                                         GdkEventButton *event)
 {
    DwImage *image = DW_IMAGE (widget);
    gint link_pressed, link_released, link_x, link_y;
+   gboolean return_val1 = FALSE, return_val2 = FALSE;
 
    link_pressed = image->pressed_link;
    Dw_image_find_link (image, x, y, &link_released, &link_x, &link_y);
@@ -416,25 +452,30 @@ static gint Dw_image_button_release (DwWidget *widget,
 
    if (link_released >= 0) {
       gtk_signal_emit (GTK_OBJECT (widget), image_signals[LINK_RELEASED],
-                       link_released, link_x, link_y, event);
-      if (link_pressed == link_released)
+                       link_released, link_x, link_y, event, &return_val1);
+      if (link_pressed == link_released) {
          gtk_signal_emit (GTK_OBJECT (widget), image_signals[LINK_CLICKED],
-                          link_released, link_x, link_y, event);
-      return TRUE;
-   } else
-      return FALSE;
+                          link_released, link_x, link_y, event, &return_val2);
+         return return_val1 || return_val2;
+      }
+      
+      return return_val1;
+   }
+
+   return FALSE;
 }
 
 
 /*
  * Standard Dw function.
  */
-static gint Dw_image_motion_notify (DwWidget *widget,
-                                    gint32 x, gint32 y,
-                                    GdkEventMotion *event)
+static gboolean Dw_image_motion_notify (DwWidget *widget,
+                                        gint32 x, gint32 y,
+                                        GdkEventMotion *event)
 {
    DwImage *image = DW_IMAGE (widget);
    gint link, link_old, link_x, link_y;
+   gboolean return_val = FALSE;
 
    if (image->tooltip)
       a_Dw_tooltip_on_motion (image->tooltip);
@@ -446,8 +487,8 @@ static gint Dw_image_motion_notify (DwWidget *widget,
    /* In case of a ISMAP image, the signal has to be emitted always */
    if (link_x != -1 || link != link_old) {
       gtk_signal_emit (GTK_OBJECT (widget), image_signals[LINK_ENTERED],
-                       link, link_x, link_y);
-      return TRUE;
+                       link, link_x, link_y, &return_val);
+      return return_val;
    } else
       return (link != -1);
 }
@@ -468,15 +509,17 @@ static void Dw_image_find_link (DwImage *image,
    *link_y = -1;
 
    if (image->map_list && image->usemap_url) {
-      *link = Dw_image_map_list_find_link (image->map_list,
-                                           image->usemap_url, x, y);
+      *link = Dw_image_map_list_find_link
+         (image->map_list, image->usemap_url,
+          x - p_Dw_style_box_offset_x (widget->style),
+          y - p_Dw_style_box_offset_y (widget->style));
       if (*link != -1)
          return;
    }
 
    if (image->ismap &&
-       (style = DW_WIDGET(image)->style) && style->link != -1) {
-      *link = style->link;
+       (style = DW_WIDGET(image)->style) && style->x_link != -1) {
+      *link = style->x_link;
       *link_x = x - p_Dw_style_box_offset_x (widget->style);
       *link_y = y - p_Dw_style_box_offset_y (widget->style);
       return;
@@ -513,7 +556,7 @@ void a_Dw_image_draw_row (DwImage *image,
    DwWidget *widget = DW_WIDGET (image);
    gint dy1, dy2;
 
-   // g_print("a_Dw_image_draw_row: x=%d y=%d\n", x, y);
+   _MSG("a_Dw_image_draw_row: x=%d y=%d\n", x, y);
    g_return_if_fail (image->buffer != NULL);
 
    if (image->scaled_buffer) {
@@ -584,9 +627,8 @@ static void Dw_image_scale (DwImage *image)
    }
 
    widget = DW_WIDGET (image);
-   w = widget->allocation.width - p_Dw_style_box_diff_width (widget->style);
-   h = widget->allocation.ascent + widget->allocation.descent -
-      p_Dw_style_box_diff_height (widget->style);
+   w = DW_WIDGET_CONTENT_WIDTH(widget);
+   h = DW_WIDGET_CONTENT_HEIGHT(widget);
 
    /* Zero or negative sizes? Ignore. */
    if (w <= 0 || h <= 0)
@@ -613,8 +655,7 @@ static void Dw_image_scale_row (DwImage *image, gint y_src)
 
    widget = DW_WIDGET (image);
    w_src = image->width;
-   w_dest =
-      widget->allocation.width - p_Dw_style_box_diff_width (widget->style);
+   w_dest = DW_WIDGET_CONTENT_WIDTH(widget);
    y_dest1 = Dw_image_scaled_y (image, y_src);
    y_dest2 = Dw_image_scaled_y (image, y_src + 1);
 
@@ -836,7 +877,8 @@ static DwIterator *Dw_image_iterator (DwWidget *widget,
 
    if (image->alt_text) {
       it = p_Dw_widget_text_iterator (widget, mask, at_end, image->alt_text);
-      it->highlight = Dw_image_iterator_highlight;
+      if (it)
+         it->highlight = Dw_image_iterator_highlight;
       return it;
    } else
       return NULL;
