@@ -11,7 +11,6 @@
 
 #include "dw_style.h"
 #include "dw_widget.h"
-#include "prefs.h"
 
 #include <gdk/gdk.h>
 #include <stdio.h>
@@ -42,6 +41,14 @@ static GHashTable *shaded_colors_table;
 
 static gint styles_num = 0;
 
+/* Used by a_Dw_style_numtostr(). */
+static const char
+   *roman_I0[] ={"I","II","III","IV","V","VI","VII","VIII","IX"},
+   *roman_I1[] ={"X","XX","XXX","XL","L","LX","LXX","LXXX","XC"},
+   *roman_I2[] ={"C","CC","CCC","CD","D","DC","DCC","DCCC","CM"},
+   *roman_I3[] ={"M","MM","MMM","MMMM"};
+
+
 static gint Dw_style_font_equal (gconstpointer v1, gconstpointer v2);
 static guint Dw_style_font_hash (gconstpointer key);
 static void Dw_style_font_remove (DwStyleFont *font);
@@ -157,16 +164,14 @@ void a_Dw_style_freeall (void)
 void a_Dw_style_init_values (DwStyle *style_attrs,
                              GdkWindow *window)
 {
-   style_attrs->link = -1;
-   style_attrs->uline = FALSE;
-   style_attrs->strike = FALSE;
-   style_attrs->SubSup = -1;
-   style_attrs->nowrap = FALSE;
+   style_attrs->x_link = -1;
+   style_attrs->text_decoration = 0;
    style_attrs->text_align = DW_STYLE_TEXT_ALIGN_LEFT;
+   style_attrs->list_style_type = DW_STYLE_LIST_STYLE_TYPE_DISC;
    style_attrs->valign = DW_STYLE_VALIGN_MIDDLE;
    style_attrs->background_color = NULL;
-   style_attrs->width = DW_STYLE_UNDEF_LENGTH;
-   style_attrs->height = DW_STYLE_UNDEF_LENGTH;
+   style_attrs->width = DW_STYLE_LENGTH_AUTO;
+   style_attrs->height = DW_STYLE_LENGTH_AUTO;
 
    a_Dw_style_box_set_val (&style_attrs->margin, 0);
    a_Dw_style_box_set_val (&style_attrs->border_width, 0);
@@ -174,9 +179,40 @@ void a_Dw_style_init_values (DwStyle *style_attrs,
    a_Dw_style_box_set_border_color (style_attrs, NULL);
    a_Dw_style_box_set_border_style (style_attrs, DW_STYLE_BORDER_NONE);
    style_attrs->border_spacing = 0;
+
+   style_attrs->display = DW_STYLE_DISPLAY_INLINE;
+   style_attrs->white_space = DW_STYLE_WHITE_SPACE_NORMAL;
 }
 
 
+/*
+ * Reset those style attributes to their standard values, which are
+ * not inherited, according to CSS.
+ */
+void a_Dw_style_reset_values (DwStyle *style_attrs,
+                              GdkWindow *window)
+{
+   style_attrs->x_link = -1;
+   style_attrs->background_color = NULL;
+
+   style_attrs->text_align = DW_STYLE_TEXT_ALIGN_LEFT; /* ??? */
+   style_attrs->valign = DW_STYLE_VALIGN_MIDDLE;
+   style_attrs->text_align_char = '.';
+   style_attrs->background_color = NULL;
+   style_attrs->width = DW_STYLE_LENGTH_AUTO;
+   style_attrs->height = DW_STYLE_LENGTH_AUTO;
+
+   a_Dw_style_box_set_val (&style_attrs->margin, 0);
+   a_Dw_style_box_set_val (&style_attrs->border_width, 0);
+   a_Dw_style_box_set_val (&style_attrs->padding, 0);
+   a_Dw_style_box_set_border_color (style_attrs, NULL);
+   a_Dw_style_box_set_border_style (style_attrs, DW_STYLE_BORDER_NONE);
+   style_attrs->border_spacing = 0;
+
+   style_attrs->display = DW_STYLE_DISPLAY_INLINE;
+   style_attrs->white_space = DW_STYLE_WHITE_SPACE_NORMAL;
+}
+
 /*
  * Return a new DwStyle, with increased reference pointer.
  */
@@ -242,22 +278,33 @@ void Dw_style_remove (DwStyle *style)
  */
 static void Dw_style_font_realize (DwStyleFont *font, gboolean try_all)
 {
-   char fontname[256], *ItalicChar;
+   char fontname[256], *style_char_1 = NULL, *style_char_2 = NULL;
 
-   ItalicChar = prefs.use_oblique ? "o" : "i";
+   switch (font->style) {
+   case DW_STYLE_FONT_STYLE_NORMAL:
+      style_char_1 = style_char_2 = "r";
+      break;
+   case DW_STYLE_FONT_STYLE_ITALIC:
+      style_char_1 = "i";
+      style_char_2 = "o";
+      break;
+   case DW_STYLE_FONT_STYLE_OBLIQUE:
+      style_char_1 = "o";
+      style_char_2 = "i";
+      break;
+   }
+   
    sprintf (fontname, "-*-%s-%s-%s-*-*-%d-*-75-75-*-*-iso8859-1",
             font->name,
-            font->bold ? "bold" : "medium",
-            font->italic ? ItalicChar : "r",
-            font->size);
+            (font->weight >= 500) ? "bold" : "medium",
+            style_char_1, font->size);
    font->font = gdk_font_load (fontname);
 
-   if (font->font == NULL && font->italic) {
+   if (font->font == NULL && font->style != DW_STYLE_FONT_STYLE_NORMAL) {
       sprintf (fontname, "-*-%s-%s-%s-*-*-%d-*-75-75-*-*-iso8859-1",
                font->name,
-               font->bold ? "bold" : "medium",
-               (*ItalicChar == 'o') ? "i" : "o",
-               font->size);
+               (font->weight >= 500) ? "bold" : "medium",
+               style_char_2, font->size);
       font->font = gdk_font_load (fontname);
    }
 
@@ -299,8 +346,8 @@ static gint Dw_style_font_equal (gconstpointer v1, gconstpointer v2)
    const DwStyleFont *font1 = (DwStyleFont*) v1, *font2 = (DwStyleFont*) v2;
 
    return (font1->size == font2->size &&
-           EQUIV (font1->bold, font2->bold) &&
-           EQUIV (font1->italic, font2->italic) &&
+           font1->weight == font2->weight &&
+           font1->style == font2->style &&
            strcmp (font1->name, font2->name) == 0);
 }
 
@@ -315,8 +362,8 @@ static guint Dw_style_font_hash (gconstpointer key)
 
    h = g_str_hash (font->name);
    h = (h << 5) - h + font->size;
-   h = (h << 5) - h + (font->bold ? 1 : 0);
-   h = (h << 5) - h + (font->italic ? 1 : 0);
+   h = (h << 5) - h + font->weight;
+   h = (h << 5) - h + font->style;
    return h;
 }
 
@@ -342,8 +389,8 @@ static DwStyleFont* Dw_style_font_new_internal (DwStyleFont *font_attrs,
    } else {
       font = g_new (DwStyleFont, 1);
       font->size = font_attrs->size;
-      font->bold = font_attrs->bold;
-      font->italic = font_attrs->italic;
+      font->weight = font_attrs->weight;
+      font->style = font_attrs->style;
       font->name = g_strdup (font_attrs->name);
       font->ref_count = 0;
 
@@ -381,7 +428,8 @@ DwStyleFont* a_Dw_style_font_new (DwStyleFont *font_attrs)
  * containing a comma separated list of names. See also
  * Dw_style_font_new_internal.
  */
-DwStyleFont* a_Dw_style_font_new_from_list (DwStyleFont *font_attrs)
+DwStyleFont* a_Dw_style_font_new_from_list (DwStyleFont *font_attrs,
+                                            gchar *default_family)
 {
    DwStyleFont *font = NULL, font_attrs2;
    char *comma, *list, *current;
@@ -408,7 +456,7 @@ DwStyleFont* a_Dw_style_font_new_from_list (DwStyleFont *font_attrs)
    g_free (list);
 
    if (font == NULL) {
-      font_attrs2.name = prefs.vw_fontname;
+      font_attrs2.name = default_family;
       font = Dw_style_font_new_internal (&font_attrs2, TRUE);
    }
 
@@ -731,6 +779,23 @@ static void Dw_style_shaded_color_remove (DwStyleShadedColor *color)
    g_free (color);
 }
 
+/*
+ * This function returns whether something may change its size, when
+ * its style changes from style1 to style2. It is mainly for
+ * optimizing style changes where only colors etc change (where FALSE
+ * would be returned), in some cases it may return TRUE, although a
+ * size change does not actually happen (e.g. when in a certain
+ * context a particular attribute is ignored).
+ */
+gboolean a_Dw_style_size_diffs (DwStyle *style1,
+                                DwStyle *style2)
+{
+   /* todo: Should for CSS implemented properly. Currently, size
+    * changes are not needed, so always FALSE is returned. See also
+    * a_Dw_widget_set_style(). */
+   return FALSE;
+}
+
 
 /* ----------------------------------------------------------------------
  *
@@ -910,3 +975,61 @@ void p_Dw_style_draw_background (GdkWindow *window,
                              intersection.width, intersection.height);
    }
 }
+
+
+/*
+ * Convert a number into a string, in a given style. Used for ordered lists.
+ */
+void a_Dw_style_numtostr (gint num,
+                          gchar *buf,
+                          gint buflen,
+                          DwStyleListStyleType list_style_tyle)
+{
+   int i3, i2, i1, i0;
+   gboolean low = FALSE;
+
+   switch(list_style_tyle){
+   case DW_STYLE_LIST_STYLE_TYPE_LOWER_ALPHA:
+      low = TRUE;
+   case DW_STYLE_LIST_STYLE_TYPE_UPPER_ALPHA:
+      i0 = num - 1;
+      i1 = i0/26; i2 = i1/26;
+      i0 %= 26;   i1 %= 26;
+      if (i2 > 26) /* more than 26*26*26=17576 elements ? */
+         sprintf(buf, "****.");
+      else if (i2)
+         sprintf(buf, "%c%c%c.", 'A'+i2-1,'A'+i1-1,'A'+i0);
+      else if (i1)
+         sprintf(buf, "%c%c.", 'A'+i1-1,'A'+i0);
+      else
+         sprintf(buf, "%c.", 'A'+i0);
+      if (low)
+         g_strdown(buf);
+      break;
+   case DW_STYLE_LIST_STYLE_TYPE_LOWER_ROMAN:
+      low = TRUE;
+   case DW_STYLE_LIST_STYLE_TYPE_UPPER_ROMAN:
+      i0 = num - 1;
+      i1 = i0/10; i2 = i1/10; i3 = i2/10;
+      i0 %= 10;   i1 %= 10;   i2 %= 10;
+      if (i3 > 4) /* more than 4999 elements ? */
+         sprintf(buf, "****.");
+      else if (i3)
+         sprintf(buf, "%s%s%s%s.", roman_I3[i3-1], roman_I2[i2-1],
+                 roman_I1[i1-1], roman_I0[i0]);
+      else if (i2)
+         sprintf(buf, "%s%s%s.", roman_I2[i2-1],
+                 roman_I1[i1-1], roman_I0[i0]);
+      else if (i1)
+         sprintf(buf, "%s%s.", roman_I1[i1-1], roman_I0[i0]);
+      else
+         sprintf(buf, "%s.", roman_I0[i0]);
+      if (low)
+         g_strdown(buf);
+      break;
+   case DW_STYLE_LIST_STYLE_TYPE_DECIMAL:
+   default:
+      g_snprintf(buf, buflen, "%d.", num);
+      break;
+   }
+}
