@@ -66,18 +66,18 @@ static void Dw_table_set_descent   (DwWidget *widget,
 static void Dw_table_draw          (DwWidget *widget,
                                     DwRectangle *area,
                                     GdkEventExpose *event);
-static gint Dw_table_button_press  (DwWidget *widget,
-                                    gint32 x,
-                                    gint32 y,
-                                    GdkEventButton *event);
-static gint Dw_table_button_release(DwWidget *widget,
-                                    gint32 x,
-                                    gint32 y,
-                                   GdkEventButton *event);
-static gint Dw_table_motion_notify (DwWidget *widget,
-                                    gint32 x,
-                                    gint32 y,
-                                    GdkEventMotion *event);
+static gboolean Dw_table_button_press  (DwWidget *widget,
+                                        gint32 x,
+                                        gint32 y,
+                                        GdkEventButton *event);
+static gboolean Dw_table_button_release(DwWidget *widget,
+                                        gint32 x,
+                                        gint32 y,
+                                        GdkEventButton *event);
+static gboolean Dw_table_motion_notify (DwWidget *widget,
+                                        gint32 x,
+                                        gint32 y,
+                                        GdkEventMotion *event);
 
 static void Dw_table_add           (DwContainer *container,
                                     DwWidget *widget);
@@ -154,7 +154,7 @@ DwWidget* a_Dw_table_new (void)
    GtkObject *object;
 
    object = gtk_object_new (DW_TYPE_TABLE, NULL);
-   DBG_CREATE (object, "DwTable");
+   DBG_OBJ_CREATE (object, "DwTable");
    return DW_WIDGET (object);
 }
 
@@ -280,24 +280,23 @@ static void Dw_table_size_request (DwWidget *widget,
    if (table->num_cols > 0) {
       a_Dw_widget_get_extremes (DW_WIDGET (table), &extremes);
 
-      if (DW_STYLE_IS_LENGTH (widget->style->width))
-         avail_width = DW_STYLE_GET_LENGTH (widget->style->width,
-                                            widget->style->font);
-      else if (DW_STYLE_IS_PERCENTAGE (widget->style->width))
+      if (DW_STYLE_IS_ABS_LENGTH (widget->style->width))
+         avail_width = DW_STYLE_ABS_LENGTH_VAL (widget->style->width);
+      else if (DW_STYLE_IS_PER_LENGTH (widget->style->width))
          /*
           * If the width is > 100%, we use 100%, this prevents ugly
           * results. (May be changed in future, when a more powerful
           * rendering is implemented, to handle fixed positions etc.,
           * as defined by CSS2.)
           */
-         avail_width = table->avail_width *
-            MIN (DW_STYLE_GET_PERCENTAGE (widget->style->width), 1.0);
+         avail_width = table->avail_width
+            * MIN (DW_STYLE_PER_LENGTH_VAL (widget->style->width), 1.0);
       else
          avail_width = table->avail_width;
 
       if (avail_width < extremes.min_width)
          avail_width = extremes.min_width;
-      if (widget->style->width == DW_STYLE_UNDEF_LENGTH &&
+      if (widget->style->width == DW_STYLE_LENGTH_AUTO &&
           avail_width > extremes.max_width)
          avail_width = extremes.max_width;
 
@@ -504,6 +503,9 @@ static void Dw_table_size_allocate (DwWidget *widget,
                child_allocation.descent =
                   cell_height - table->baseline[row];
                break;
+            default:
+               g_assert_not_reached();
+               break;
             }
 
             a_Dw_widget_size_allocate (child, &child_allocation);
@@ -582,8 +584,8 @@ static void Dw_table_draw (DwWidget *widget,
 
    offx = p_Dw_style_box_offset_x (widget->style);
    offy = p_Dw_style_box_offset_y (widget->style);
-   width =
-      widget->allocation.width - p_Dw_style_box_diff_width (widget->style);
+   width = DW_WIDGET_CONTENT_WIDTH(widget);
+
    for (i = 0; i < table->num_rows; i++)
       if (table->row_style[i])
          p_Dw_widget_draw_box (widget, table->row_style[i], area,
@@ -604,7 +606,7 @@ static void Dw_table_draw (DwWidget *widget,
                 Dw_widget_x_world_to_viewport (child, child->allocation.x) - 1,
                 Dw_widget_y_world_to_viewport (child, child->allocation.y) - 1,
                 child->allocation.width + 2,
-                child->allocation.ascent + child->allocation.descent + 2);
+                DW_WIDGET_HEIGHT(child) + 2);
 #endif
             a_Dw_widget_draw (child, &child_area, event);
          }
@@ -618,7 +620,7 @@ static void Dw_table_draw (DwWidget *widget,
                        Dw_widget_y_world_to_viewport (widget,
                                                       widget->allocation.y),
                        widget->allocation.width,
-                       widget->allocation.ascent + widget->allocation.descent);
+                       DW_WIDGET_HEIGHT(widget));
 #endif
 }
 
@@ -644,9 +646,9 @@ static DwIterator *Dw_table_construct_iterator (DwTable *table,
 static DwIterator *Dw_table_construct_last_iterator (DwTable *table,
                                                      int *char_pos)
 {
-   DwIterator *it = NULL;
    int n;
-   DwTableChild *child;
+   DwIterator *it = NULL;
+   DwTableChild *child = NULL;
 
    *char_pos = 0;
    n = table->num_children -1;
@@ -760,10 +762,10 @@ static gboolean Dw_table_send_selection_event (DwTable *table,
 /*
  * Standard Dw function.
  */
-static gint Dw_table_button_press (DwWidget *widget,
-                                   gint32 x,
-                                   gint32 y,
-                                   GdkEventButton *event)
+static gboolean Dw_table_button_press (DwWidget *widget,
+                                       gint32 x,
+                                       gint32 y,
+                                       GdkEventButton *event)
 {
    return Dw_table_send_selection_event (DW_TABLE (widget),
                                          a_Selection_button_press, x, y,
@@ -773,10 +775,10 @@ static gint Dw_table_button_press (DwWidget *widget,
 /*
  * Standard Dw function.
  */
-static gint Dw_table_button_release (DwWidget *widget,
-                                     gint32 x,
-                                     gint32 y,
-                                     GdkEventButton *event)
+static gboolean Dw_table_button_release (DwWidget *widget,
+                                         gint32 x,
+                                         gint32 y,
+                                         GdkEventButton *event)
 {
    return Dw_table_send_selection_event (DW_TABLE (widget),
                                          a_Selection_button_release, x, y,
@@ -787,10 +789,10 @@ static gint Dw_table_button_release (DwWidget *widget,
 /*
  * Standard Dw function.
  */
-static gint Dw_table_motion_notify (DwWidget *widget,
-                                    gint32 x,
-                                    gint32 y,
-                                    GdkEventMotion *event)
+static gboolean Dw_table_motion_notify (DwWidget *widget,
+                                        gint32 x,
+                                        gint32 y,
+                                        GdkEventMotion *event)
 {
    if (event && (event->state & GDK_BUTTON1_MASK))
       return Dw_table_send_selection_event (DW_TABLE (widget),
@@ -1235,9 +1237,9 @@ static void Dw_table_sub_get_extremes (DwTableSub *sub)
                colspan = sub->table->children[start_n]->data.cell.colspan;
 
                /* Adjust width argument of the cell */
-               if (DW_STYLE_IS_LENGTH (child->style->width)) {
-                  col_width = DW_STYLE_GET_LENGTH (child->style->width,
-                                                   child->style->font);
+               if (DW_STYLE_IS_ABS_LENGTH (child->style->width)) {
+                  col_width =
+                     DW_STYLE_ABS_LENGTH_VAL (child->style->width);
                   if (col_width < child_extremes.min_width)
                      col_width = child_extremes.min_width;
                   child_extremes.min_width = col_width;
@@ -1246,8 +1248,8 @@ static void Dw_table_sub_get_extremes (DwTableSub *sub)
                   DEBUG_MSG (DEBUG_EXTR_LEVEL, "         [%p] following "
                              "adjusted by %d pixels:\n",
                              sub->table, col_width);
-               } else if (DW_STYLE_IS_PERCENTAGE (child->style->width)) {
-                  percentage = DW_STYLE_GET_PERCENTAGE (child->style->width);
+               } else if (DW_STYLE_IS_PER_LENGTH (child->style->width)) {
+                  percentage = DW_STYLE_PER_LENGTH_VAL (child->style->width);
                   sub->percentage = MAX (sub->percentage, percentage);
                   sub->use_percentage = USE_PERCENTAGE_ALL;
                   DEBUG_MSG (DEBUG_EXTR_LEVEL,
@@ -1290,9 +1292,8 @@ static void Dw_table_sub_get_extremes (DwTableSub *sub)
       /* Adjust width argument of the table (fixed, percentages
          later in the code). */
       widget = DW_WIDGET (sub->table);
-      if (DW_STYLE_IS_LENGTH (widget->style->width)) {
-         col_width = DW_STYLE_GET_LENGTH (widget->style->width,
-                                          widget->style->font);
+      if (DW_STYLE_IS_ABS_LENGTH (widget->style->width)) {
+         col_width = DW_STYLE_ABS_LENGTH_VAL (widget->style->width);
          if (col_width < sub->span_extremes.min_width)
             col_width = sub->span_extremes.min_width;
          sub->span_extremes.min_width = col_width;
@@ -1676,7 +1677,7 @@ static DwIterator *Dw_table_iterator (DwWidget *widget,
    it->it.compare = p_Dw_iterator_compare_std_int;
    it->it.free = p_Dw_iterator_free_std;
    it->it.highlight = p_Dw_iterator_highlight_std;
-   it->it.scroll_to = p_Dw_iterator_scroll_to_std;
+   it->it.get_allocation = p_Dw_iterator_get_allocation_std;
 
    if (at_end) {
       it->it.content.type = DW_CONTENT_END;
