@@ -20,6 +20,7 @@
 #include <sys/time.h>
 #include <fcntl.h>
 
+#include "msg.h"
 #include "list.h"
 #include "misc.h"
 #include "dillo.h"
@@ -176,7 +177,7 @@ static gint Interface_sens_idle_func(BrowserWindow *bw)
                      !bw->nav_expecting);
    gtk_widget_set_sensitive(bw->forw_button, forw_sensitive);
 
-   bw->sens_idle_tag = 0;
+   bw->sens_idle_id = 0;
    return FALSE;
 }
 
@@ -185,10 +186,9 @@ static gint Interface_sens_idle_func(BrowserWindow *bw)
  */
 void a_Interface_set_button_sens(BrowserWindow *bw)
 {
-   if (bw->sens_idle_tag != 0)
-      return;
-   bw->sens_idle_tag = gtk_idle_add(
-                          (GtkFunction)Interface_sens_idle_func, bw);
+   if (bw->sens_idle_id == 0)
+      bw->sens_idle_id = gtk_idle_add(
+                             (GtkFunction)Interface_sens_idle_func, bw);
 }
 
 /*
@@ -253,10 +253,10 @@ void a_Interface_add_url(BrowserWindow *bw, const DilloUrl *Url, gint Flags)
    }
 
    /* test:
-   g_print("Urls:\n");
+   MSG("Urls:\n");
    for (i = 0; i < bw->NumPageUrls; i++)
-      g_print("%s\n", bw->PageUrls[i].Url);
-   g_print("---\n");
+      MSG("%s\n", bw->PageUrls[i].Url);
+   MSG("---\n");
    */
 }
 
@@ -282,18 +282,29 @@ static gboolean Interface_quit(GtkWidget *widget, BrowserWindow *bw)
       gtk_widget_destroy(bw->quit_dialog_window);
    if (bw->findtext_dialog_window != NULL)
       gtk_widget_destroy(bw->findtext_dialog_window);
+   if (bw->search_dialog_window != NULL)
+      gtk_widget_destroy(bw->search_dialog_window);
    if (bw->question_dialog_window != NULL)
       gtk_widget_destroy(bw->question_dialog_window);
 
+   if (bw->menu_popup.over_page)
+      gtk_widget_destroy(bw->menu_popup.over_page);
+   if (bw->menu_popup.over_link)
+      /* this also destroys menu_popup.over_image */
+      gtk_widget_destroy(bw->menu_popup.over_link);
    if (bw->menu_popup.over_back)
       gtk_widget_destroy(bw->menu_popup.over_back);
    if (bw->menu_popup.over_forw)
       gtk_widget_destroy(bw->menu_popup.over_forw);
+   if (bw->menu_popup.over_bug)
+      gtk_widget_destroy(bw->menu_popup.over_bug);
    if (bw->menu_popup.url)
       a_Url_free(bw->menu_popup.url);
+   if (bw->menu_popup.url2)
+      a_Url_free(bw->menu_popup.url2);
 
-   if (bw->sens_idle_tag)
-      gtk_idle_remove(bw->sens_idle_tag);
+   if (bw->sens_idle_id)
+      gtk_idle_remove(bw->sens_idle_id);
 
    for (i = 0; i < num_bw; i++)
       if (browser_window[i] == bw) {
@@ -377,11 +388,11 @@ static void Interface_set_button_accel(GtkButton *button,
 }
 
 /*
- * Create the "NEW" button and its location-entry.
+ * Create the "NEW" button, its location-entry and the search button.
  */
 static GtkWidget *Interface_locbar_new(BrowserWindow *bw)
 {
-   GtkWidget *hbox, *toolbar;
+   GtkWidget *hbox, *toolbar_l, *toolbar_r;
 
    hbox = gtk_hbox_new(FALSE, 0);
 
@@ -389,12 +400,17 @@ static GtkWidget *Interface_locbar_new(BrowserWindow *bw)
    bw->location = gtk_entry_new();
    gtk_signal_connect(GTK_OBJECT(bw->location), "activate",
                       (GtkSignalFunc) a_Interface_entry_open_url, bw);
-   toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
-   gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar), GTK_RELIEF_NONE);
-   GTK_WIDGET_UNSET_FLAGS (toolbar, GTK_CAN_FOCUS);
+   gtk_widget_add_accelerator(GTK_WIDGET(bw->location), "grab_focus",
+                              bw->accel_group, GDK_u,
+                              GDK_CONTROL_MASK, GTK_ACCEL_LOCKED);
+
+   /* left toolbar (Clear url) */
+   toolbar_l = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
+   gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar_l), GTK_RELIEF_NONE);
+   GTK_WIDGET_UNSET_FLAGS (toolbar_l, GTK_CAN_FOCUS);
 
    bw->clear_url_button = gtk_toolbar_append_item(
-                             GTK_TOOLBAR(toolbar),
+                             GTK_TOOLBAR(toolbar_l),
                              NULL, "Clear the url-box!", "Toolbar/New",
                              Interface_pixmap_new(bw->main_window, s_new_xpm),
                              NULL, NULL);
@@ -402,11 +418,28 @@ static GtkWidget *Interface_locbar_new(BrowserWindow *bw)
                              GTK_SIGNAL_FUNC (Interface_entry_clear),
                              GTK_OBJECT(bw->location));
 
-   gtk_box_pack_start(GTK_BOX(hbox), toolbar, FALSE, FALSE, 0);
-   gtk_widget_show(toolbar);
+   /* right toolbar (Search) */
+   toolbar_r = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
+   gtk_toolbar_set_button_relief(GTK_TOOLBAR(toolbar_r), GTK_RELIEF_NONE);
+   GTK_WIDGET_UNSET_FLAGS (toolbar_r, GTK_CAN_FOCUS);
+
+   bw->search_button = gtk_toolbar_append_item(
+                          GTK_TOOLBAR(toolbar_r),
+                          NULL, "Search the Web", "toolbar/New",
+                          Interface_pixmap_new(bw->main_window, search_xpm),
+                          GTK_SIGNAL_FUNC(a_Interface_search_dialog),
+                          bw);
+   gtk_widget_add_accelerator(GTK_WIDGET(bw->search_button), "clicked",
+                              bw->accel_group, GDK_s,
+                              GDK_CONTROL_MASK, GTK_ACCEL_LOCKED);
+   gtk_box_pack_start(GTK_BOX(hbox), toolbar_l, FALSE, FALSE, 0);
+   gtk_widget_show(toolbar_l);
    gtk_box_pack_start(GTK_BOX(hbox), bw->location, TRUE, TRUE, 0);
    gtk_widget_show(bw->location);
+   gtk_box_pack_start(GTK_BOX(hbox), toolbar_r, FALSE, FALSE, 0);
+   gtk_widget_show(toolbar_r);
    gtk_widget_show(hbox);
+
    return (hbox);
 }
 
@@ -468,6 +501,8 @@ static GtkWidget *Interface_toolbar_new(BrowserWindow *bw, gint label)
                           Interface_pixmap_new(bw->main_window,
                                                s ? s_reload_xpm : reload_xpm),
                           (GtkSignalFunc) a_Commands_reload_callback, bw);
+   Interface_set_button_accel(GTK_BUTTON(bw->reload_button), GDK_r,
+                              GDK_CONTROL_MASK, bw->accel_group);
 
    /* save button */
    bw->save_button = gtk_toolbar_append_item(
@@ -498,8 +533,8 @@ static GtkWidget *Interface_toolbar_new(BrowserWindow *bw, gint label)
    gtk_widget_set_sensitive(bw->bookmarks_button, TRUE);
    Interface_set_button_accel(GTK_BUTTON(bw->bookmarks_button), GDK_semicolon,
                               0, bw->accel_group);
-   Interface_set_button_accel(GTK_BUTTON(bw->bookmarks_button), GDK_B,
-                              GDK_MOD1_MASK, bw->accel_group);
+   Interface_set_button_accel(GTK_BUTTON(bw->bookmarks_button), GDK_b,
+                              GDK_CONTROL_MASK, bw->accel_group);
    gtk_signal_connect(GTK_OBJECT(bw->bookmarks_button), "button-press-event",
                       GTK_SIGNAL_FUNC(a_Commands_navpress_callback), bw);
 
@@ -538,6 +573,7 @@ void Interface_toggle_panel(BrowserWindow *bw, gint toggle)
       g_slist_foreach(bw->PanelHandles, (GFunc)gtk_widget_hide, NULL);
       gtk_widget_hide(bw->status_box);
       gtk_widget_show (bw->full_screen_off_button);
+      gtk_widget_grab_focus(GTK_BIN(bw->docwin)->child);
    } else {
       g_slist_foreach(bw->PanelHandles, (GFunc)gtk_widget_show, NULL);
       gtk_widget_show(bw->status_box);
@@ -570,6 +606,8 @@ static void Interface_browser_window_customize(BrowserWindow *bw)
       gtk_widget_hide(bw->clear_url_button);
    if ( !prefs.show_url )
       gtk_widget_hide(bw->location);
+   if ( !prefs.show_search )
+      gtk_widget_hide(bw->search_button);
    if ( !prefs.show_progress_box )
       gtk_widget_hide(bw->progress_box);
 
@@ -615,7 +653,7 @@ static void Interface_key_press_handler(GtkWidget *widget,
       a_Commands_findtext_callback(NULL, bw);
       break;
    default:
-      // g_print(">> Key pressed!\n");
+      _MSG(">> Key pressed!\n");
       break;
    }
 }
@@ -661,7 +699,7 @@ static void Interface_add_full_screen_button (BrowserWindow *bw, GtkBox *box)
 BrowserWindow *
 a_Interface_browser_window_new(gint width, gint height, guint32 xid)
 {
-   GtkWidget *box1, *hbox,
+   GtkWidget *box1, *hbox, *button, *label,
              *progbox, *toolbar, *handlebox, *menubar, *locbox, *pixmap;
    BrowserWindow *bw;
    char buf[64];
@@ -801,6 +839,11 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
    gtk_box_pack_start(GTK_BOX(box1), bw->docwin, TRUE, TRUE, 0);
    gtk_widget_show(bw->docwin);
 
+   /* todo (GTK2): this call is a bit of a workaround to missing focus
+    * between the location box and the docwin. It's fixed in GTK2:
+    * http://mail.gnome.org/archives/gtk-list/2003-June/msg00307.html  */
+   gtk_widget_grab_focus(GTK_BIN(bw->docwin)->child);
+
    /* Full screen mode via double click is done in two ways: First,
     * a feature of the selection is used, since in complex pages,
     * getting events back to the viewport is quite difficult. Second,
@@ -808,6 +851,11 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
    a_Selection_set_dclick_callback(
       GTK_DW_VIEWPORT(GTK_BIN(GTK_BIN(bw->docwin)->child)->child)->selection,
       (void(*)(gpointer))Interface_full_screen_callback, bw);
+   /* Selection requires an owner widget */
+   a_Selection_set_owner(
+      GTK_DW_VIEWPORT(GTK_BIN(GTK_BIN(bw->docwin)->child)->child)->selection,
+      GTK_BIN(bw->docwin)->child);
+
    gtk_signal_connect_object_after(GTK_OBJECT(GTK_BIN(bw->docwin)->child),
                                    "button_press_event",
                                    GTK_SIGNAL_FUNC(Interface_click_callback),
@@ -828,6 +876,8 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
    gtk_signal_connect_object
       (GTK_OBJECT (bw->full_screen_off_button), "clicked",
        GTK_SIGNAL_FUNC (Interface_full_screen_callback), (gpointer)bw);
+   Interface_set_button_accel(GTK_BUTTON(bw->full_screen_off_button), GDK_h,
+                              GDK_CONTROL_MASK, bw->accel_group);
 
    /* Catch key_press event */
    gtk_signal_connect(GTK_OBJECT(GTK_BIN(bw->docwin)->child),
@@ -839,10 +889,30 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
    /* status widget */
    bw->status = a_Dw_gtk_statuslabel_new("");
    gtk_misc_set_alignment(GTK_MISC(bw->status), 0.0, 0.5);
-   bw->status_box = gtk_hbox_new(FALSE, 0);
+   /*  status widget for HTML errors.
+    *  Note: the "clicked" signal is hooked with the linkblock later */
+   button = bw->status_bug_meter = gtk_button_new();
+   GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
+   hbox = gtk_hbox_new(FALSE, 0);
+   pixmap = Interface_pixmap_new(bw->main_window, mini_bug_xpm);
+   gtk_object_set_data(GTK_OBJECT (button), "bug", pixmap);
+   gtk_box_pack_start(GTK_BOX(hbox), pixmap, FALSE, FALSE, 0);
+   pixmap = Interface_pixmap_new(bw->main_window, mini_ok_xpm);
+   gtk_object_set_data(GTK_OBJECT (button), "ok", pixmap);
+   gtk_box_pack_start(GTK_BOX(hbox), pixmap, FALSE, FALSE, 0);
+   gtk_widget_show (pixmap);
+   label = gtk_label_new("");
+   gtk_object_set_data(GTK_OBJECT (button), "label", label);
+   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+   gtk_container_add (GTK_CONTAINER (button), hbox);
+   gtk_widget_show(hbox);
+   gtk_tooltips_set_tip (tooltips, button, "Show HTML bugs", "");
 
+   bw->status_box = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(bw->status_box), bw->status, TRUE, TRUE, 2);
    gtk_widget_show(bw->status);
+   gtk_box_pack_start(GTK_BOX(bw->status_box), button, FALSE, FALSE, 0);
+   gtk_widget_show(button);
 
    Interface_add_full_screen_button (bw, GTK_BOX (bw->status_box));
 
@@ -856,12 +926,18 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
    bw->pagemarks_menuitem = NULL;
    bw->pagemarks_menu = NULL;
    bw->pagemarks_last = NULL;
-   bw->menu_popup.over_page = a_Menu_popup_op_new(bw);
+   bw->viewbugs_menuitem = NULL;
+
+   /* the image menu is created first because it is used by the link menu */
+   bw->menu_popup.over_image = a_Menu_popup_oi_new(bw);
    bw->menu_popup.over_link = a_Menu_popup_ol_new(bw);
+   bw->menu_popup.over_page = a_Menu_popup_op_new(bw);
+   bw->menu_popup.over_bug  = a_Menu_popup_ob_new(bw);
    bw->menu_popup.over_back = NULL;
    bw->menu_popup.over_forw = NULL;
    bw->menu_popup.url = NULL;
-   bw->sens_idle_tag = 0;
+   bw->menu_popup.url2 = NULL;
+   bw->sens_idle_id = 0;
 
    bw->CursorType = -1;
 
@@ -888,9 +964,12 @@ a_Interface_browser_window_new(gint width, gint height, guint32 xid)
    bw->findtext_dialog_window = NULL;
    bw->findtext_dialog_check = NULL;
    bw->findtext_dialog_entry = NULL;
+   bw->search_dialog_window = NULL;
+   bw->search_dialog_entry = NULL;
    bw->question_dialog_window = NULL;
    bw->question_dialog_data = NULL;
    bw->viewsource_window = NULL;
+   bw->pagebugs_window = NULL;
 
    /* now that the bw is made, let's customize it.. */
    Interface_browser_window_customize(bw);
@@ -951,11 +1030,40 @@ void a_Interface_msg(BrowserWindow *bw, const char *format, ... )
       va_start(argp, format);
       vsnprintf(msg, 1024, format, argp);
       va_end(argp);
-      gtk_label_set(GTK_LABEL(bw->status), msg);
+      gtk_label_set_text(GTK_LABEL(bw->status), msg);
       bw->status_is_link = 0;
    }
 }
 
+/*
+ * Update the bug-meter button for detected page errors.
+ */
+void a_Interface_bug_meter_update(BrowserWindow *bw, gint num_err)
+{
+   static char msg[64];
+   gpointer label, bug, ok;
+
+   if ( bw ) {
+      label = gtk_object_get_data(GTK_OBJECT(bw->status_bug_meter), "label");
+      if (num_err < 2) {
+         bug = gtk_object_get_data(GTK_OBJECT(bw->status_bug_meter), "bug");
+         ok = gtk_object_get_data(GTK_OBJECT(bw->status_bug_meter), "ok");
+         if (num_err == 0) {
+            gtk_widget_hide(bug);
+            gtk_widget_hide(label);
+            gtk_widget_show(ok);
+         } else {
+            gtk_widget_show(bug);
+            gtk_widget_show(label);
+            gtk_widget_hide(ok);
+         }
+         gtk_widget_queue_resize (GTK_WIDGET (bw->status_bug_meter));
+      }
+      g_snprintf(msg, 64, " %d", num_err);
+      gtk_label_set_text(GTK_LABEL(label), msg);
+   }
+}
+
 /*
  * Called from `destroy' callback in Interface_make_*_dialog
  */
@@ -1039,7 +1147,7 @@ static void
 
    Cfn = (s = a_Misc_escape_chars(fn, "% ")) ? s : fn;
    g_string_sprintf(UrlStr, "file:%s", Cfn);
-   url = a_Url_new(UrlStr->str, NULL, 0, 0);
+   url = a_Url_new(UrlStr->str, NULL, 0, 0, 0);
    a_Nav_push(bw, url);
    a_Url_free(url);
    g_string_free(UrlStr, TRUE);
@@ -1048,28 +1156,59 @@ static void
    gtk_widget_destroy(bw->openfile_dialog_window);
 }
 
+/* 
+ * Returns a newly allocated string holding a search url generated from 
+ * a string of keywords (separarated by blanks) and prefs.search_url.
+ * Any '"' or '+' in the keywords are escaped. 
+ */
+static gchar *Interface_make_search_url(const gchar *str)
+{
+   gchar *p, *keys, *new_str;
+
+   keys = g_new(gchar, 3*strlen(str)+1);
+
+   for (p = keys; *str; str++) {
+      switch(*str) {
+      case ' ':
+         *p++ = '+';
+         break;
+      case '"':
+         *p++ = '%';
+         *p++ = '2';
+         *p++ = '2';
+         break;
+      case '+':
+         *p++ = '%';
+         *p++ = '2';
+         *p++ = 'B';
+         break;
+      default:
+         *p++ = *str;
+      }
+   }
+   *p = '\0';
+
+   new_str = g_strdup_printf(prefs.search_url, keys);
+   g_free(keys);
+
+   return new_str;
+}
+
 /*
- * Open an URL specified in the location entry, or in the open URL dialog.
+ * Open an url string.
  * The URL is not sent "as is", illegal chars are ripped out,
  * then it's fully parsed by a_Url_new().
  */
-void a_Interface_entry_open_url(GtkWidget *widget, BrowserWindow *bw)
+static void Interface_open_url_string(gchar *text, BrowserWindow *bw)
 {
-   gchar *text, *new_text;
+   gchar *new_text;
    DilloUrl *url;
-   GtkEntry *entry;
 
-   /* entry = { bw->location | bw->open_dialog_entry } */
-   entry = GTK_ENTRY(widget == bw->location ? widget : bw->open_dialog_entry);
-   text = gtk_entry_get_text(entry);
-
-   DEBUG_MSG(1, "entry_open_url %s\n", text);
-
-   if ( text && *text ) {
+   if (text && *text) {
       /* Filter URL string */
       new_text = a_Url_string_strip_delimiters(text);
 
-      url = a_Url_new(new_text, NULL, 0, 0);
+      url = a_Url_new(new_text, NULL, 0, 0, 0);
       if (url) {
          a_Nav_push(bw, url);
          a_Url_free(url);
@@ -1077,6 +1216,26 @@ void a_Interface_entry_open_url(GtkWidget *widget, BrowserWindow *bw)
       g_free(new_text);
    }
 
+   /* let the rendered area have focus */
+   gtk_widget_grab_focus(GTK_BIN(bw->docwin)->child);
+}
+
+/*
+ * Open an URL specified in the location entry, or in the open URL dialog.
+ */
+void a_Interface_entry_open_url(GtkWidget *widget, BrowserWindow *bw)
+{
+   gchar *text;
+   GtkEntry *entry;
+
+   /* entry = { bw->location | bw->open_dialog_entry } */
+   entry = GTK_ENTRY(widget == bw->location ? widget : bw->open_dialog_entry);
+   text = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
+
+   DEBUG_MSG(1, "entry_open_url %s\n", text);
+   Interface_open_url_string(text, bw);
+   g_free(text);
+
    if (bw->open_dialog_window != NULL)
       gtk_widget_hide(bw->open_dialog_window);
 }
@@ -1331,19 +1490,33 @@ static void Interface_file_save_link(GtkWidget *widget, BrowserWindow *bw)
 {
    const gchar *name;
    const DilloUrl *url;
+   gchar *cmd, *buf;
    FILE *out;
+   gint buf_size;
 
    name = gtk_file_selection_get_filename(
              GTK_FILE_SELECTION(bw->save_link_dialog_window));
    url  = a_Menu_popup_get_url(bw);
 
-   if ( strlen(name) && (out = fopen(name, "w")) != NULL ) {
-      DilloWeb *Web = a_Web_new(url);
-      Web->bw = bw;
-      Web->stream = out;
-      Web->flags |= WEB_Download;
-      /* todo: keep track of this client */
-      a_Capi_open_url(Web, Interface_save_callback, Web);
+   if (!a_Capi_get_buf(url, &buf, &buf_size)) {
+      /* Not cached, ask the downloads server to get it */
+      cmd = g_strdup_printf(
+               "<dpi cmd='download' url='%s' destination='%s'>\n",
+               URL_STR(url), name);
+      a_Capi_dpi_send_cmd(NULL, bw, cmd, "downloads", 1);
+      g_free(cmd);
+
+   } else {
+      /* Cached! Save from the cache */
+      if ( strlen(name) && (out = fopen(name, "w")) != NULL ) {
+         DilloWeb *Web = a_Web_new(url);
+         Web->bw = bw;
+         Web->stream = out;
+         Web->flags |= WEB_Download;
+         /* todo: keep track of this client */
+         a_Capi_open_url(Web, Interface_save_callback, Web);
+      } else
+         g_printerr("Error trying to save: %s\n", name);
    }
 
    gtk_widget_destroy(bw->save_link_dialog_window);
@@ -1375,7 +1548,7 @@ void a_Interface_save_dialog(GtkWidget *widget, BrowserWindow *bw)
          "save_dialog", "Dillo", "Dillo: Save URL as File...",
          (GtkSignalFunc) Interface_file_save_url, (void *)bw );
    }
-   url = a_Url_new(a_Interface_get_location_text(bw), NULL, 0, 0);
+   url = a_Url_new(a_Interface_get_location_text(bw), NULL, 0, 0, 0);
    SuggestedName = Interface_make_save_name(url);
    gtk_file_selection_set_filename(
       GTK_FILE_SELECTION(bw->save_dialog_window), SuggestedName);
@@ -1472,6 +1645,45 @@ void a_Interface_findtext_dialog(BrowserWindow *bw)
       gdk_window_raise(bw->findtext_dialog_window->window);
 }
 
+/*
+ * Use the search dialog's entry to feed a web search engine.
+ */
+static void Interface_search_callback(GtkWidget *widget, BrowserWindow *bw)
+{
+   gchar *keyw, *url_str;
+
+   keyw = gtk_editable_get_chars(GTK_EDITABLE(bw->search_dialog_entry), 0, -1);
+   if (keyw) { 
+      url_str = Interface_make_search_url(keyw);
+      Interface_open_url_string(url_str, bw);
+      g_free(url_str);
+      g_free(keyw);
+   }
+   if (bw->search_dialog_window != NULL)
+      gtk_widget_hide(bw->search_dialog_window);
+}
+
+/*
+ * Show the dialog interface for web search engine.
+ */
+void a_Interface_search_dialog(GtkWidget *widget, BrowserWindow *bw)
+{
+   if (!bw->search_dialog_window) {
+      Interface_make_dialog(&(bw->search_dialog_window),
+         "search_dialog", "Dillo", "Dillo: Search the Web",
+         &(bw->search_dialog_entry), "", NULL, NULL,
+         "Search", (GtkSignalFunc)Interface_search_callback, (void *)bw);
+      if (prefs.transient_dialogs)
+         gtk_window_set_transient_for(GTK_WINDOW(bw->search_dialog_window),
+                                      GTK_WINDOW(bw->main_window));
+   }
+
+   if (!GTK_WIDGET_VISIBLE(bw->search_dialog_window))
+      gtk_widget_show(bw->search_dialog_window);
+   else
+      gdk_window_raise(bw->search_dialog_window->window);
+}
+
 /*
  * This signal callback adjusts the position of a menu.
  * It's useful for very long menus.
@@ -1551,6 +1763,71 @@ void a_Interface_message_window(const char *title, const char *format, ... )
    gtk_widget_show(window);
 }
 
+/*
+ * A general purpose window for long text display.
+ */
+void a_Interface_text_window (GtkWidget **text_widget,
+                              gchar *title, gchar *wm_class,
+                              gchar *buf, gint buf_size,
+                              gint xsize_max, gint ysize_max)
+{
+   gint xsize, ysize;
+   GtkWidget *window, *box1, *button, *scrolled_window, *text;
+
+   if (*text_widget)
+      gtk_widget_destroy (*text_widget);
+
+   /* -RL :: This code is adapted from testgtk. */
+   if ( !*text_widget ) {
+      window = *text_widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+      gtk_window_set_wmclass(GTK_WINDOW(window), wm_class, "Dillo");
+      gtk_widget_set_name (window, "text window");
+      xsize = (prefs.width < xsize_max) ? prefs.width : xsize_max;
+      ysize = (prefs.height < ysize_max) ? prefs.height : ysize_max;
+      gtk_widget_set_usize (window, xsize, ysize);
+      gtk_window_set_policy (GTK_WINDOW(window), TRUE, TRUE, FALSE);
+
+      gtk_signal_connect (GTK_OBJECT (window), "destroy",
+                          GTK_SIGNAL_FUNC(gtk_widget_destroyed),
+                          text_widget);
+
+      gtk_window_set_title (GTK_WINDOW (window), title);
+      gtk_container_border_width (GTK_CONTAINER (window), 0);
+
+      box1 = gtk_vbox_new (FALSE, 0);
+      gtk_container_add (GTK_CONTAINER (window), box1);
+      gtk_widget_show (box1);
+
+      scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+      gtk_box_pack_start (GTK_BOX (box1), scrolled_window, TRUE, TRUE, 0);
+      gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+                                      GTK_POLICY_NEVER,
+                                      GTK_POLICY_ALWAYS);
+      gtk_widget_show (scrolled_window);
+
+      text = gtk_text_new (NULL, NULL);
+      gtk_text_set_editable (GTK_TEXT (text), FALSE);
+      gtk_container_add (GTK_CONTAINER (scrolled_window), text);
+      gtk_widget_show (text);
+
+      gtk_text_freeze (GTK_TEXT (text));
+      gtk_text_insert (GTK_TEXT (text), NULL, NULL, NULL, buf, buf_size);
+      gtk_text_thaw (GTK_TEXT (text));
+
+      button = gtk_button_new_with_label ("close");
+      gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+                                 GTK_SIGNAL_FUNC(gtk_widget_destroy),
+                                 GTK_OBJECT (window));
+      gtk_box_pack_start (GTK_BOX (box1), button, FALSE, FALSE, 0);
+      GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+      gtk_widget_grab_default (button);
+      gtk_widget_show (button);
+   }
+
+   if (!GTK_WIDGET_VISIBLE (*text_widget))
+      gtk_widget_show (*text_widget);
+}
+
 /*
  * Places win1 in a way that it does not, or as less as possible, cover win2.
  */
