@@ -2,7 +2,7 @@
  * File: nav.c
  *
  * Copyright (C) 1999 James McCollough <jamesm@gtwn.net>
- * Copyright (C) 2000, 2001, 2002 Jorge Arellano Cid <jcid@inf.utfsm.cl>
+ * Copyright (C) 2000, 2001, 2002 Jorge Arellano Cid <jcid@dillo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,6 +14,7 @@
 
 #include <stdio.h>
 #include <gtk/gtk.h>
+#include "msg.h"
 #include "list.h"
 #include "nav.h"
 #include "history.h"
@@ -155,7 +156,7 @@ static void Nav_open_url(BrowserWindow *bw, const DilloUrl *url, gint offset)
    DilloWeb *Web;
    gboolean ForceReload = (URL_FLAGS(url) & URL_E2EReload);
 
-   g_print("Nav_open_url: Url=>%s<\n", URL_STR_(url));
+   MSG("Nav_open_url: Url=>%s<\n", URL_STR_(url));
 
    /* Get the url of the current page */
    if ( a_Nav_stack_ptr(bw) != -1 )
@@ -165,8 +166,11 @@ static void Nav_open_url(BrowserWindow *bw, const DilloUrl *url, gint offset)
     * (the strcmp check is necessary because of redirections) */
    if (old_url &&
        !strcmp(URL_STR(old_url), a_Interface_get_location_text(bw))) {
-      old_url->scrolling_position =
-        a_Dw_gtk_scrolled_window_get_scrolling_position(
+      old_url->scrolling_position_x =
+        a_Dw_gtk_scrolled_window_get_scrolling_position_x(
+           GTK_DW_SCROLLED_WINDOW(bw->docwin));
+      old_url->scrolling_position_y =
+        a_Dw_gtk_scrolled_window_get_scrolling_position_y(
            GTK_DW_SCROLLED_WINDOW(bw->docwin));
    }
 
@@ -276,6 +280,19 @@ void a_Nav_push(BrowserWindow *bw, const DilloUrl *url)
    Nav_open_url(bw, url, 0);
 }
 
+/*
+ * Same as a_Nav_push() but in a new window.
+ */
+void a_Nav_push_nw(BrowserWindow *bw, const DilloUrl *url)
+{
+   gint width, height;
+   BrowserWindow *newbw;
+
+   gdk_window_get_size (bw->main_window->window, &width, &height);
+   newbw = a_Interface_browser_window_new(width, height, 0);
+   a_Nav_push(newbw, url);
+}
+
 /*
  * Wraps a_Nav_push to match 'DwPage->link' function type
  */
@@ -332,8 +349,7 @@ void a_Nav_jump_callback(GtkWidget *widget, gpointer client_data, gint NewBw)
    idx = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT (widget), "nav_idx"));
    if (idx >= 0 && idx < a_Nav_stack_size(bw)) {
       if (NewBw == 1) {
-         a_Menu_popup_set_url(bw, a_History_get_url(NAV_IDX(bw,idx)));
-         a_Commands_open_link_nw_callback(widget, bw);
+         a_Nav_push_nw(bw, a_History_get_url(NAV_IDX(bw,idx)));
       } else {
          Nav_open_url(bw, a_History_get_url(NAV_IDX(bw,idx)),
                       idx - a_Nav_stack_ptr(bw));
