@@ -1,7 +1,7 @@
 /*
  * File: pixmaps.h
  *
- * Copyright (C) 2000, 2001 Jorge Arellano Cid <jcid@inf.utfsm.cl>
+ * Copyright (C) 2000, 2001 Jorge Arellano Cid <jcid@dillo.org>
  * Design by John Grantham, Dipl.-Designer; http://www.grantham.de/
  *
  * This program is free software; you can redistribute it and/or modify
@@ -1370,6 +1370,37 @@ static char * s_new_xpm[] = {
 " .56   65. ",
 "  .     .  "};
 
+/* XPM */
+static char * search_xpm[] = {
+"14 16 11 1",
+"       c None",
+".      c #000000",
+"+      c #EEEEEE",
+"[      c #EE0000",
+"}      c #CC0000",
+"|      c #BB0000",
+"1      c #AA0000",
+"2      c #880000",
+"3      c #660000",
+"4      c #440000",
+"5      c #330000",
+"   .....      ",    
+"  .     .     ",    
+" .  +    .    ",    
+".  +++    .   ",    
+".   +     .   ",    
+".         .   ",    
+".         .   ",    
+" .       .    ",    
+"  .     .     ",    
+"   ...545     ",    
+"       424    ",    
+"       313    ",    
+"        2|2   ",    
+"        2}2   ",    
+"         2}2  ",    
+"         11   "};   
+
 /* XPM */
 static char * full_screen_on_xpm[] = {
 "15 15 2 1",
@@ -1412,4 +1443,47 @@ static char * full_screen_off_xpm[] = {
 " ............. ",
 "               "};
 
+/* XPM */
+static char * mini_bug_xpm[] = {
+"16 16 5 1",
+"       c None s None",
+".      c black",
+"X      c #c0c0c0",
+"o      c #808080",
+"#      c white"
+"                ",
+"                ",
+"      .   .     ",
+"       ...      ",
+"     X.....X    ",
+"     o.#...o    ",
+"    o.#...o.o   ",
+"    ..#oXo...   ",
+"   .....X.....  ",
+"    ....X....   ",
+"   .o...X...o.  ",
+"     ...X...    ",
+"    .X..X..X.   ",
+"       .o.      ",
+"                ",
+"                "};
+
+/* XPM */
+static char * mini_ok_xpm[] = {
+"11 11 2 1",
+"@ c #000000",
+"_ s none m none c none",
+"__________@",
+"_________@@",
+"________@@@",
+"_______@@@_",
+"_@@___@@@__",
+"@@@__@@@___",
+"@@@_@@@____",
+"@@@@@@_____",
+"@@@@@______",
+"@@@@_______",
+"_@@________"
+};
+
 #endif /* __PIXMAPS_H__ */
