@@ -10,9 +10,7 @@
  */
 
 #include <config.h>
-#ifdef HAVE_LIBPNG
-
-#undef VERBOSE
+#ifdef ENABLE_PNG
 
 #include <stdio.h>
 #include <string.h>
@@ -20,29 +18,31 @@
 
 #include <zlib.h>
 
-#ifdef HAVE_PNG_H
-#include <png.h>
-#else
+#ifdef HAVE_LIBPNG_PNG_H
 #include <libpng/png.h>
+#else
+#include <png.h>
 #endif
 
 #include <gtk/gtk.h>
+#include "msg.h"
 #include "image.h"
 #include "web.h"
 #include "cache.h"
 #include "dicache.h"
 #include "prefs.h"
 
+#define DEBUG_LEVEL 6
+#include "debug.h"
+
 enum prog_state {
    IS_finished, IS_init, IS_nextdata
 };
 
-#if defined(VERBOSE)
 static char *prog_state_name[] =
 {
    "IS_finished", "IS_init", "IS_nextdata"
 };
-#endif
 
 /*
  * This holds the data that must be saved between calls to this module.
@@ -112,7 +112,7 @@ void Png_error_handling(png_structp png_ptr, png_const_charp msg)
 {
    DilloPng *png;
 
-   g_print("Png_error_handling: %s\n", msg);
+   DEBUG_MSG(6, "Png_error_handling: %s\n", msg);
    png = png_get_error_ptr(png_ptr);
 
    png->error = 1;
@@ -131,19 +131,17 @@ Png_datainfo_callback(png_structp png_ptr, png_infop info_ptr)
    guint i;
    double gamma;
 
-#if defined(VERBOSE)
-   g_print("Png_datainfo_callback:\n");
-#endif
+   DEBUG_MSG(5, "Png_datainfo_callback:\n");
 
    png = png_get_progressive_ptr(png_ptr);
    g_return_if_fail (png != NULL);
 
    png_get_IHDR(png_ptr, info_ptr, &png->width, &png->height,
                 &bit_depth, &color_type, &interlace_type, NULL, NULL);
-#if defined(VERBOSE)
-   g_print("Png_datainfo_callback: png->width  = %ld\n", png->width);
-   g_print("Png_datainfo_callback: png->height = %ld\n", png->height);
-#endif
+
+   DEBUG_MSG(5, "Png_datainfo_callback: png->width  = %ld\n"
+             "Png_datainfo_callback: png->height = %ld\n",
+             png->width, png->height);
 
    /* we need RGB/RGBA in the end */
    if (color_type == PNG_COLOR_TYPE_PALETTE && bit_depth <= 8) {
@@ -190,11 +188,10 @@ Png_datainfo_callback(png_structp png_ptr, png_infop info_ptr)
    png->channels = png_get_channels(png_ptr, info_ptr);
 
    /* init Dillo specifics */
-#if defined(VERBOSE)
-   g_print("Png_datainfo_callback: rowbytes = %d\n", png->rowbytes);
-   g_print("Png_datainfo_callback: width    = %ld\n", png->width);
-   g_print("Png_datainfo_callback: height   = %ld\n", png->height);
-#endif
+   DEBUG_MSG(5, "Png_datainfo_callback: rowbytes = %d\n"
+             "Png_datainfo_callback: width    = %ld\n"
+             "Png_datainfo_callback: height   = %ld\n",
+             png->rowbytes, png->width, png->height);
 
    png->image_data = (guchar *) g_malloc(png->rowbytes * png->height);
    png->row_pointers = (guchar **) g_malloc(png->height * sizeof(guchar *));
@@ -219,9 +216,7 @@ static void
    if (!new_row)                /* work to do? */
       return;
 
-#if defined(VERBOSE)
-   g_print("Png_datarow_callback: row_num = %ld\n", row_num);
-#endif
+   DEBUG_MSG(5, "Png_datarow_callback: row_num = %ld\n", row_num);
 
    png = png_get_progressive_ptr(png_ptr);
 
@@ -271,8 +266,8 @@ static void
         break;
      }
    default:
-      g_print("Png_datarow_callback: unexpected number of channels = %d\n",
-              png->channels);
+      MSG("Png_datarow_callback: unexpected number of channels = %d\n",
+          png->channels);
       abort();
    }
 }
@@ -282,9 +277,7 @@ static void
 {
    DilloPng *png;
 
-#if defined(VERBOSE)
-   g_print("Png_dataend_callback:\n");
-#endif
+   DEBUG_MSG(5, "Png_dataend_callback:\n");
 
    png = png_get_progressive_ptr(png_ptr);
    png->state = IS_finished;
@@ -337,9 +330,7 @@ static void Png_callback(int Op, CacheClient_t *Client)
    /* Let's make some sound if we have been called with no data */
    g_return_if_fail ( Client->Buf != NULL && Client->BufSize > 0 );
 
-#if defined(VERBOSE)
-   g_print("Png_callback BufSize = %d\n", Client->BufSize);
-#endif
+   DEBUG_MSG(5, "Png_callback BufSize = %d\n", Client->BufSize);
 
    /* Keep local copies so we don't have to pass multiple args to
     * a number of functions. */
@@ -348,9 +339,8 @@ static void Png_callback(int Op, CacheClient_t *Client)
 
    /* start/resume the FSM here */
    while (png->state != IS_finished && DATASIZE) {
-#if defined(VERBOSE)
-      g_print("State = %s\n", prog_state_name[png->state]);
-#endif
+      DEBUG_MSG(5, "State = %s\n", prog_state_name[png->state]);
+
       switch (png->state) {
       case IS_init:
          if (DATASIZE < 8) {
@@ -443,13 +433,11 @@ DwWidget *a_Png_image(const char *Type, void *Ptr, CA_Callback_t *Call,
    DilloWeb *web = Ptr;
    DICacheEntry *DicEntry;
 
-#if defined(VERBOSE)
-   g_print("a_Png_image: Type = %s\n", Type);
-   g_print("a_Png_image: libpng - Compiled %s; using %s.\n",
-           PNG_LIBPNG_VER_STRING, png_libpng_ver);
-   g_print("a_Png_image: zlib   - Compiled %s; using %s.\n",
-           ZLIB_VERSION, zlib_version);
-#endif
+   DEBUG_MSG(5, "a_Png_image: Type = %s\n"
+             "a_Png_image: libpng - Compiled %s; using %s.\n"
+             "a_Png_image: zlib   - Compiled %s; using %s.\n",
+             Type, PNG_LIBPNG_VER_STRING, png_libpng_ver,
+             ZLIB_VERSION, zlib_version);
 
    if ( !web->Image )
       web->Image = a_Image_new(0, 0, NULL, NULL, prefs.bg_color);
@@ -474,4 +462,4 @@ DwWidget *a_Png_image(const char *Type, void *Ptr, CA_Callback_t *Call,
    return DW_WIDGET (web->Image->dw);
 }
 
-#endif
+#endif /* ENABLE_PNG */
