@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <string.h>       /* for strchr */
 #include <fcntl.h>
+#include <unistd.h>
 #include <locale.h>       /* for setlocale */
 #include "prefs.h"
 #include "colors.h"
@@ -62,13 +63,17 @@ static const struct {
    { "show_menubar", DRC_TOKEN_SHOW_MENUBAR },
    { "show_clear_url", DRC_TOKEN_SHOW_CLEAR_URL },
    { "show_url", DRC_TOKEN_SHOW_URL },
+   { "show_search", DRC_TOKEN_SHOW_SEARCH },
    { "show_progress_box", DRC_TOKEN_SHOW_PROGRESS_BOX },
    { "fullwindow_start", DRC_TOKEN_FULLWINDOW_START },
    { "transient_dialogs", DRC_TOKEN_TRANSIENT_DIALOGS },
    { "vw_fontname", DRC_TOKEN_VW_FONT },
    { "fw_fontname", DRC_TOKEN_FW_FONT },
    { "generate_submit", DRC_TOKEN_GENERATE_SUBMIT },
-   { "enterpress_forces_submit", DRC_TOKEN_ENTERPRESS_FORCES_SUBMIT }
+   { "enterpress_forces_submit", DRC_TOKEN_ENTERPRESS_FORCES_SUBMIT },
+   { "search_url", DRC_TOKEN_SEARCH_URL },
+   { "show_msg", DRC_TOKEN_SHOW_MSG },
+   { "use_old_parser", DRC_TOKEN_USE_OLD_PARSER }
 };
 
 static const guint n_symbols = sizeof (symbols) / sizeof (symbols[0]);
@@ -80,6 +85,7 @@ static const guint n_symbols = sizeof (symbols) / sizeof (symbols[0]);
  */
 static guint Prefs_parser(GScanner *scanner)
 {
+   gint st;
    guint symbol;
 
    /* expect a valid symbol */
@@ -103,20 +109,13 @@ static guint Prefs_parser(GScanner *scanner)
 
    /* assign value and exit successfully */
    switch (symbol) {
-   case DRC_TOKEN_GEOMETRY: {
-      gchar *ptr, *str = scanner->value.v_string;
-      gint width, height;
-
-      if ( (ptr = strchr(str, 'x')) && (width = strtol(str,NULL,10)) &&
-           (height = strtol(++ptr,NULL,10)) ){
-         prefs.width = width;
-         prefs.height = height;
-      }
-     }
-     break;
+   case DRC_TOKEN_GEOMETRY:
+      a_Misc_parse_geometry(scanner->value.v_string, &prefs.xpos, &prefs.ypos,
+                            &prefs.width, &prefs.height);
+      break;
    case DRC_TOKEN_PROXY:
       a_Url_free(prefs.http_proxy);
-      prefs.http_proxy = a_Url_new(scanner->value.v_string, NULL, 0, 0);
+      prefs.http_proxy = a_Url_new(scanner->value.v_string, NULL, 0, 0, 0);
       break;
    case DRC_TOKEN_NOPROXY:
       g_free(prefs.no_proxy);
@@ -125,19 +124,19 @@ static guint Prefs_parser(GScanner *scanner)
       break;
    case DRC_TOKEN_LINK_COLOR:
       prefs.link_color = a_Color_parse(scanner->value.v_string,
-                                       prefs.link_color);
+                                       prefs.link_color, &st);
       break;
    case DRC_TOKEN_VISITED_COLOR:
       prefs.visited_color = a_Color_parse(scanner->value.v_string,
-                                          prefs.visited_color);
+                                          prefs.visited_color, &st);
       break;
    case DRC_TOKEN_TEXT_COLOR:
       prefs.text_color = a_Color_parse(scanner->value.v_string,
-                                       prefs.text_color);
+                                       prefs.text_color, &st);
       break;
    case DRC_TOKEN_BG_COLOR:
       prefs.bg_color = a_Color_parse(scanner->value.v_string,
-                                     prefs.bg_color);
+                                     prefs.bg_color, &st);
       break;
    case DRC_TOKEN_ALLOW_WHITE_BG:
       prefs.allow_white_bg = (strcmp(scanner->value.v_string, "YES") == 0);
@@ -165,7 +164,7 @@ static guint Prefs_parser(GScanner *scanner)
       break;
    case DRC_TOKEN_HOME:
       a_Url_free(prefs.home);
-      prefs.home = a_Url_new(scanner->value.v_string, NULL, 0, 0);
+      prefs.home = a_Url_new(scanner->value.v_string, NULL, 0, 0, 0);
       break;
    case DRC_TOKEN_SHOW_TOOLTIP:
       prefs.show_tooltip = (strcmp(scanner->value.v_string, "YES") == 0);
@@ -209,6 +208,9 @@ static guint Prefs_parser(GScanner *scanner)
    case DRC_TOKEN_SHOW_URL:
       prefs.show_url = (strcmp(scanner->value.v_string, "YES") == 0);
       break;
+   case DRC_TOKEN_SHOW_SEARCH:
+      prefs.show_search = (strcmp(scanner->value.v_string, "YES") == 0);
+      break;
    case DRC_TOKEN_SHOW_PROGRESS_BOX:
       prefs.show_progress_box = (strcmp(scanner->value.v_string, "YES") == 0);
       break;
@@ -233,6 +235,18 @@ static guint Prefs_parser(GScanner *scanner)
       prefs.enterpress_forces_submit =
          (strcmp(scanner->value.v_string, "YES") == 0);
       break;
+   case DRC_TOKEN_SEARCH_URL:
+      g_free(prefs.search_url);
+      prefs.search_url = g_strdup(scanner->value.v_string);
+      break;
+   case DRC_TOKEN_SHOW_MSG:
+      prefs.show_msg =
+         (strcmp(scanner->value.v_string, "YES") == 0);
+      break;
+   case DRC_TOKEN_USE_OLD_PARSER:
+      prefs.use_old_parser =
+         (strcmp(scanner->value.v_string, "YES") == 0);
+      break;
    default:
       break;   /* Not reached */
    }
@@ -261,7 +275,7 @@ static gint Prefs_load(void)
    /* Specifies the chars which can be used in identifiers */
    scanner->config->cset_identifier_nth = (
       G_CSET_a_2_z
-      "~-_:&%#/.0123456789"
+      "~+-_:&%#/.0123456789"
       G_CSET_A_2_Z
       G_CSET_LATINS   /*??? I don't know if we need these two */
       G_CSET_LATINC   /*??? */
@@ -320,6 +334,7 @@ static gint Prefs_load(void)
 
    /* finish parsing */
    g_scanner_destroy(scanner);
+   close(fd);
    return PARSE_OK;
 }
 
@@ -329,6 +344,8 @@ void a_Prefs_init(void)
 
    prefs.width = D_GEOMETRY_DEFAULT_WIDTH;
    prefs.height = D_GEOMETRY_DEFAULT_HEIGHT;
+   prefs.xpos = D_GEOMETRY_DEFAULT_XPOS;
+   prefs.ypos = D_GEOMETRY_DEFAULT_YPOS;
    prefs.http_proxy = NULL;
    prefs.no_proxy = NULL;
    prefs.no_proxy_vec = NULL;
@@ -337,7 +354,7 @@ void a_Prefs_init(void)
    prefs.bg_color = DW_COLOR_DEFAULT_BGND;
    prefs.text_color = DW_COLOR_DEFAULT_BLACK;
    prefs.use_oblique = FALSE;
-   prefs.home = a_Url_new(DILLO_HOME, NULL, 0, 0);
+   prefs.home = a_Url_new(DILLO_HOME, NULL, 0, 0, 0);
    prefs.allow_white_bg = TRUE;
    prefs.force_my_colors = FALSE;
    prefs.force_visited_color = FALSE;
@@ -357,6 +374,7 @@ void a_Prefs_init(void)
    prefs.show_menubar=TRUE;
    prefs.show_clear_url=TRUE;
    prefs.show_url=TRUE;
+   prefs.show_search=TRUE;
    prefs.show_progress_box=TRUE;
    prefs.fullwindow_start=FALSE;
    prefs.transient_dialogs=FALSE;
@@ -364,6 +382,9 @@ void a_Prefs_init(void)
    prefs.fw_fontname = g_strdup("courier");
    prefs.generate_submit = FALSE;
    prefs.enterpress_forces_submit = FALSE;
+   prefs.search_url = g_strdup("http://www.google.com/search?q=%s");
+   prefs.show_msg = TRUE;
+   prefs.use_old_parser = FALSE;
 
    /* this locale stuff is to avoid parsing problems with float numbers */
    old_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
@@ -388,4 +409,5 @@ void a_Prefs_freeall(void)
    g_free(prefs.fw_fontname);
    g_free(prefs.vw_fontname);
    a_Url_free(prefs.home);
+   g_free(prefs.search_url);
 }
