@@ -15,13 +15,13 @@
  */
 
 #include "selection.h"
-#include "commands.h"
+#include "strbuf.h"
 #include "dw_gtk_viewport.h"
 #include <stdio.h>
 #include <string.h>
-#include <gtk/gtksignal.h>
+#include <gtk/gtk.h>
 
-/*#define DEBUG_LEVEL 1*/
+/*#define DEBUG_LEVEL 2*/
 #include "debug.h"
 
 static void Selection_reset_selection          (Selection *selection);
@@ -60,6 +60,9 @@ Selection *a_Selection_new (void)
    selection->link = NULL;
 
    selection->dclick_callback = NULL;
+   selection->callback_data = NULL;
+
+   selection->owner = NULL;
    return selection;
 }
 
@@ -78,6 +81,14 @@ void a_Selection_set_dclick_callback (Selection *selection,
    selection->callback_data = data;
 }
 
+/*
+ * Set the widget that owns the selection.
+ */
+void a_Selection_set_owner(Selection *selection, gpointer owner)
+{
+   selection->owner = owner;
+   a_Selection_init_selection(owner);
+}
 
 void a_Selection_reset (Selection *selection)
 {
@@ -122,14 +133,14 @@ gint a_Selection_button_press  (Selection *selection,
                                 gboolean within_content)
 {
    DwWidget *it_widget = it->widget;
-   gboolean ret = FALSE;
+   gboolean ret = FALSE, dummy;
 
-   DEBUG_MSG (1, "PRESS%s(%s, %d, %d): %s -> ...\n",
+   DEBUG_MSG (3, "PRESS%s(%s, %d, %d): %s -> ...\n",
               (event && event->type == GDK_2BUTTON_PRESS) ? "2" : "",
               a_Dw_iterator_text (it), char_pos, link,
               selection_state_descr[selection->selection_state]);
 
-#if defined (DEBUG_LEVEL) && DEBUG_LEVEL >= 1
+#if defined (DEBUG_LEVEL) && 1 >= DEBUG_LEVEL
    a_Dw_widget_print_tree (GTK_DW_VIEWPORT(it->widget->viewport)->child);
 #endif
 
@@ -150,7 +161,8 @@ gint a_Selection_button_press  (Selection *selection,
          /* link handling */
          if (event) {
             gtk_signal_emit_by_name (GTK_OBJECT (it_widget),
-                                     "link_pressed", link, -1, -1, event);
+                                     "link_pressed", link, -1, -1, event,
+                                     &dummy);
             Selection_reset_link (selection);
             selection->link_state = SELECTION_LINK_PRESSED;
             selection->link_button = event->button;
@@ -164,6 +176,8 @@ gint a_Selection_button_press  (Selection *selection,
                   Selection_correct_char_pos (selection->link, char_pos);
                selection->link_number = link;
             }
+            /* We do not return the value of the signal function (dummy),
+             * but we do actually process this event. */
             ret = TRUE;
          }
       } else {
@@ -190,7 +204,7 @@ gint a_Selection_button_press  (Selection *selection,
       }
    }
 
-   DEBUG_MSG (1, "  ... -> %s, processed = %s\n",
+   DEBUG_MSG (3, "  ... -> %s, processed = %s\n",
               selection_state_descr[selection->selection_state],
               ret ? "TRUE" : "FALSE");
    return ret;
@@ -205,9 +219,9 @@ gint a_Selection_button_release (Selection *selection,
                                  gboolean within_content)
 {
    DwWidget *it_widget = it->widget;
-   gboolean ret = FALSE;
+   gboolean ret = FALSE, dummy;
 
-   DEBUG_MSG (1, "RELEASE(%s, %d, %d): %s -> ...\n",
+   DEBUG_MSG (3, "RELEASE(%s, %d, %d): %s -> ...\n",
               a_Dw_iterator_text (it), char_pos, link,
               selection_state_descr[selection->selection_state]);
 
@@ -217,13 +231,15 @@ gint a_Selection_button_release (Selection *selection,
       ret = TRUE;
       if (link != -1)
          gtk_signal_emit_by_name (GTK_OBJECT (it_widget),
-                                  "link_released", link, -1, -1, event);
+                                  "link_released", link, -1, -1, event,
+                                  &dummy);
 
       /* The link where the user clicked the mouse button? */
       if (link == selection->link_number) {
          Selection_reset_link (selection);
          gtk_signal_emit_by_name (GTK_OBJECT (it_widget),
-                                  "link_clicked", link, -1, -1, event);
+                                  "link_clicked", link, -1, -1, event,
+                                  &dummy);
       } else {
          if (event->button == 1)
             /* Reset links and switch to selection mode. The selection
@@ -249,7 +265,7 @@ gint a_Selection_button_release (Selection *selection,
       }
    }
 
-   DEBUG_MSG (1, "  ... -> %s, processed = %s\n",
+   DEBUG_MSG (3, "  ... -> %s, processed = %s\n",
               selection_state_descr[selection->selection_state],
               ret ? "TRUE" : "FALSE");
    return ret;
@@ -263,7 +279,7 @@ gint a_Selection_button_motion (Selection *selection,
                                 GdkEventButton *event,
                                 gboolean within_content)
 {
-   DEBUG_MSG (1, "MOTION (%s, %d, %d): %s -> ...\n",
+   DEBUG_MSG (3, "MOTION (%s, %d, %d): %s -> ...\n",
               a_Dw_iterator_text (it), char_pos, link,
               selection_state_descr[selection->selection_state]);
 
@@ -279,7 +295,7 @@ gint a_Selection_button_motion (Selection *selection,
       Selection_adjust_selection (selection, it, char_pos);
    }
 
-   DEBUG_MSG (1, "  ... -> %s, processed = TRUE\n",
+   DEBUG_MSG (3, "  ... -> %s, processed = TRUE\n",
               selection_state_descr[selection->selection_state]);
    return TRUE;
 }
@@ -311,7 +327,7 @@ static void Selection_switch_link_to_selection (Selection *selection,
 
       Selection_highlight (selection, TRUE);
 
-      DEBUG_MSG (1, "   after switching: from (%s, %d) to (%s, %d)\n",
+      DEBUG_MSG (2, "   after switching: from (%s, %d) to (%s, %d)\n",
                  a_Dw_ext_iterator_text (selection->from),
                  selection->from_char,
                  a_Dw_ext_iterator_text (selection->to), selection->to_char);
@@ -354,7 +370,7 @@ static void Selection_adjust_selection (Selection *selection,
       /* Here, cmp_old and cmp_new are equivalent and != 0. */
       cmp_diff = a_Dw_ext_iterator_compare (new_to, selection->to);
 
-      DEBUG_MSG (1, "Selection_adjust_selection: cmp_old = cmp_new = %d, "
+      DEBUG_MSG (2, "Selection_adjust_selection: cmp_old = cmp_new = %d, "
                  "cmp_diff = %d\n", cmp_old, cmp_diff);
 
       if (cmp_old * cmp_diff > 0) {
@@ -395,7 +411,7 @@ static void Selection_adjust_selection (Selection *selection,
    if (brute_highlighting)
       Selection_highlight (selection, TRUE);
 
-   DEBUG_MSG (1, "   selection now from (%s, %d) to (%s, %d)\n",
+   DEBUG_MSG (2, "   selection now from (%s, %d) to (%s, %d)\n",
               a_Dw_ext_iterator_text (selection->from), selection->from_char,
               a_Dw_ext_iterator_text (selection->to), selection->to_char);
 }
@@ -438,7 +454,7 @@ static void  Selection_highlight0 (gboolean fl,
    gboolean start;
 
    if (from && to) {
-      DEBUG_MSG (1, "  %shighlighting from %s / %d to %s / %d\n",
+      DEBUG_MSG (2, "  %shighlighting from %s / %d to %s / %d\n",
                  fl ? "" : "un", a_Dw_ext_iterator_text (from),
                  from_char, a_Dw_ext_iterator_text (to), to_char);
       cmp = a_Dw_ext_iterator_compare (from, to);
@@ -473,7 +489,7 @@ static void  Selection_highlight0 (gboolean fl,
          if (i->content.type == DW_CONTENT_TEXT) {
             if (fl) {
                if (start) {
-                  DEBUG_MSG (1, "    highlighting %s from %d to %d\n",
+                  DEBUG_MSG (2, "    highlighting %s from %d to %d\n",
                              a_Dw_ext_iterator_text (i), a_char,
                              strlen (i->content.data.text) + 1);
                   a_Dw_ext_iterator_highlight (i, a_char,
@@ -482,12 +498,12 @@ static void  Selection_highlight0 (gboolean fl,
                                                DW_HIGHLIGHT_SELECTION);
                } else if (cmp == 0) {
                   /* the end */
-                  DEBUG_MSG (1, "    highlighting %s from %d to %d\n",
+                  DEBUG_MSG (2, "    highlighting %s from %d to %d\n",
                              a_Dw_ext_iterator_text (i), 0, b_char);
                   a_Dw_ext_iterator_highlight (i, 0, b_char,
                                                DW_HIGHLIGHT_SELECTION);
                } else {
-                  DEBUG_MSG (1, "    highlighting %s from %d to %d\n",
+                  DEBUG_MSG (2, "    highlighting %s from %d to %d\n",
                              a_Dw_ext_iterator_text (i), 0,
                              strlen (i->content.data.text) + 1);
                   a_Dw_ext_iterator_highlight (i, 0,
@@ -496,7 +512,7 @@ static void  Selection_highlight0 (gboolean fl,
                                                DW_HIGHLIGHT_SELECTION);
                }
             } else {
-               DEBUG_MSG (1, "    unhighlighting %s\n",
+               DEBUG_MSG (2, "    unhighlighting %s\n",
                           a_Dw_ext_iterator_text (i));
                a_Dw_ext_iterator_unhighlight (i, DW_HIGHLIGHT_SELECTION);
             }
@@ -513,26 +529,24 @@ static void Selection_copy(Selection *selection)
    DwExtIterator *a, *b, *i;
    gint cmp, a_char, b_char;
    gboolean start;
-   gchar *sel = g_strdup("");
-   gchar *txt;
    gchar *tmp;
+   Strbuf_t *strbuf;
 
    if (selection->from && selection->to) {
+      strbuf = a_Strbuf_new ();
+
       cmp = a_Dw_ext_iterator_compare (selection->from, selection->to);
       if (cmp == 0) {
          if (selection->from->content.type == DW_CONTENT_TEXT) {
             si = selection->from->stack[selection->from->stack_top];
-            if (selection->from_char < selection->to_char) {
-               txt = g_strndup(si->content.data.text + selection->from_char,
+            if (selection->from_char < selection->to_char)
+               tmp = g_strndup(si->content.data.text + selection->from_char,
                                selection->to_char - selection->from_char);
-            } else {
-               txt = g_strndup(si->content.data.text + selection->to_char,
+            else
+               tmp = g_strndup(si->content.data.text + selection->to_char,
                                selection->from_char - selection->to_char);
-            }
-            tmp = g_strconcat(sel,txt,NULL);
-            g_free(sel);
-            g_free(txt);
-            sel = tmp;
+            a_Strbuf_append (strbuf, tmp);
+            g_free (tmp);
          }
       } else {
          if (cmp < 0) {
@@ -553,34 +567,29 @@ static void Selection_copy(Selection *selection)
             si = i->stack[i->stack_top];
             switch (si->content.type) {
             case DW_CONTENT_TEXT:
-               if (start)
-                  txt = g_strndup(si->content.data.text + a_char,
+               if (start) {
+                  tmp = g_strndup(si->content.data.text + a_char,
                                   strlen (i->content.data.text) - a_char);
-               else if (cmp == 0)
+                  a_Strbuf_append (strbuf, tmp);
+                  g_free (tmp);
+               } else if (cmp == 0) {
                   /* the end */
-                  txt = g_strndup(si->content.data.text,
-                                  b_char);
-               else
-                  txt = g_strndup(si->content.data.text,
-                                  strlen (i->content.data.text));
-               tmp = g_strconcat(sel,txt,NULL);
-               g_free(sel);
-               g_free(txt);
-               sel = tmp;
-               if (si->content.space && cmp != 0) {
-                  tmp = g_strconcat(sel," ",NULL);
-                  g_free(sel);
-                  sel = tmp;
-               }
+                  tmp = g_strndup(si->content.data.text, b_char);
+                  a_Strbuf_append (strbuf, tmp);
+                  g_free (tmp);
+               } else
+                  a_Strbuf_append (strbuf, si->content.data.text);
+
+               if (si->content.space && cmp != 0)
+                  a_Strbuf_append (strbuf, " ");
+
                break;
 
             case DW_CONTENT_BREAK:
                if (si->content.data.break_space > 0)
-                  tmp = g_strconcat(sel,"\n\n",NULL);
+                  a_Strbuf_append (strbuf, "\n\n");
                else
-                  tmp = g_strconcat(sel,"\n",NULL);
-               g_free(sel);
-               sel = tmp;
+                  a_Strbuf_append (strbuf, "\n");
                break;
             default:
                /* Make pedantic compilers happy. Especially
@@ -590,7 +599,74 @@ static void Selection_copy(Selection *selection)
          }
          a_Dw_ext_iterator_free (i);
       }
+      a_Selection_set_selection(selection->owner, a_Strbuf_chars(strbuf));
+      a_Strbuf_free (strbuf);
    }
-   a_Commands_set_selection(sel);
-   g_free(sel);
 }
+
+
+/* GENERAL SELECTION STUFF */
+
+/*
+ * Local data
+ */
+static char *selection = NULL;
+
+/*
+ * Sets the widget to copy the selection from.
+ */
+void a_Selection_init_selection(GtkWidget *widget)
+{
+   gtk_signal_connect(GTK_OBJECT(widget), "selection_clear_event",
+      GTK_SIGNAL_FUNC(a_Selection_clear_selection_callback), NULL);
+   gtk_signal_connect(GTK_OBJECT(widget), "selection_get",
+      GTK_SIGNAL_FUNC(a_Selection_give_selection_callback), NULL);
+   gtk_selection_add_target(widget, GDK_SELECTION_PRIMARY,
+      GDK_SELECTION_TYPE_STRING, 1);
+}
+
+/*
+ * Sets the selection, and associates it with the current widget for
+ * copying.
+ */
+void a_Selection_set_selection(GtkWidget *widget, gchar *str)
+{
+   //g_print("a_Selection_set_selection\n  old: %s\n", Selection);
+   if (gtk_selection_owner_set(widget,
+                               GDK_SELECTION_PRIMARY,
+                               GDK_CURRENT_TIME)) {
+      /* --EG: Why would it fail? (this question points to that the 
+       * GTK+ example tests the return value.) */
+
+      /* As the clear-selection callback is not being called automatically
+       * before setting the new value, it'll be cleared here.  --Jcid
+       * todo: this selection code needs a revision */
+      g_free(selection);
+      selection = g_strdup(str);
+   }
+}
+
+/*
+ * Callback for Paste (when another application wants the selection)
+ */
+void a_Selection_give_selection_callback(GtkWidget *widget,
+        GtkSelectionData *data, guint info, guint time)
+{
+   gtk_selection_data_set(data, GDK_SELECTION_TYPE_STRING,
+                          8, selection, strlen(selection));
+}
+
+/*
+ * Clear selection
+ */
+gint a_Selection_clear_selection_callback(GtkWidget *widget,
+                                         GdkEventSelection *event)
+{
+   //g_print("a_Selection_clear_selection_callback\n");
+   g_free(selection);
+   selection = NULL;
+
+   /* here we should un-highlight the selected text */
+   return TRUE;
+}
+
